
package MotorGrafico.GestorPrincipal;


import java.awt.Dimension;
import java.awt.Graphics;

import javax.swing.JFrame;
import javax.swing.JOptionPane;



import MotorGrafico.Estados.StateMachine;
import MotorGrafico.Utilidades.GestorUsuarios;
import MotorGrafico.Utilidades.GestorXML;


public class GestorPrincipal {

	private static GestorPrincipal miGestorPrincipal;	
	
	private StateMachine stateMachine;
	private GameCanvas gameCanvas;	
	
	private boolean fin;
	private boolean iniciado;
	
	private GestorPrincipal(){	
		fin = false;
		iniciado = false;
	}	
	
	public static GestorPrincipal getGestorPrincipal(){
		if(miGestorPrincipal == null){
			miGestorPrincipal = new GestorPrincipal();
		}
		return miGestorPrincipal;
	}
	
	public void iniciarJuego(int id,String usuario){
		//Comprobamos que el usuario sea correcto
		boolean usuarioCorrecto = GestorUsuarios.getGestorUsuarios().crearUsuarioActual(id, usuario);
		if(usuarioCorrecto){
			//Creamos la pantalla
			gameCanvas = new GameCanvas();
			//Creamos al maquina estados
			stateMachine = new StateMachine();		
			iniciado = true; 
		}else{
			error("User not valid.");
		}
				
	}
	
	//Cierra el juego si se da un error de conexin
	public void error(String msg){		
		if(!fin){			
			fin = true;						
			//Mostramos un panel de error
			JOptionPane.showMessageDialog(new JFrame(), msg, "ERROR",
			        JOptionPane.ERROR_MESSAGE);
			cerrarJuego();
			System.exit(1);
		}				
	}
	
	public void cerrarJuego(){
		if(!fin && iniciado){
			fin = true;
			gameCanvas.pararJuego();
		}
	}	
	
	
	//Cambia el estado de la maquina de estados
	public void cambiarEstado(int nuevoEstado){
		stateMachine.setState(nuevoEstado);
	}	
	
	//Devuelve la dimension de la pantalla
	public Dimension getDimensionPantalla(){		
		String s = GestorXML.getGestorXML().obtenerValorConfiguracion("ventana", "ancho");
		int ancho = Integer.parseInt(s);
		
		s = GestorXML.getGestorXML().obtenerValorConfiguracion("ventana", "alto");
		int alto = Integer.parseInt(s);
		
		return new Dimension(ancho,alto);
	}

	//Actualizamos la mquina estados
	public void update() {		
		if(stateMachine != null){
			stateMachine.update();
		}		
	}
	//Renderizamos
	public void render(Graphics g){
		if(stateMachine != null){
			stateMachine.render(g);
		}		
	}
	
}
