package MotorGrafico.InterfazUsuario;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;

import MotorGrafico.Controladores.ControladorRaton;
import MotorGrafico.Sonido.GestorSonido;





public class BotonInterfaz extends UIObjectInteractive {	

	private String texto;	
	private boolean mouseOver;
	private int size;
		
	//Constructor
	public BotonInterfaz(double posX, double posY, String id,String texto,int size) {
		super(posX, posY, id);
		this.texto = texto;
		this.size = size;
	}
	
	//Estilos para configurar los grficos
	private void setEstiloNormal(Graphics g){
		Font f = new Font("Showcard Gothic", Font.PLAIN, size);
		g.setFont(f);
		g.setColor(Color.CYAN);
	}
	
	private void setEstiloMouseOver(Graphics g){			
		Font f = new Font("Showcard Gothic", Font.BOLD, size);
		g.setFont(f);
		g.setColor(Color.RED);
	}	
	
	
	//Comprueba si el puntero del raton esta encima del texto
	private void comprobarMouseOver() {
		if(rectangle !=null){
			boolean estadoPrevio = mouseOver;
			//Obtenemos la posicin del raton
			double mouseX = ControladorRaton.getControladorRaton().getPosicionX();
			double mouseY = ControladorRaton.getControladorRaton().getPosicionY();			
			
			//Comprobamos si el puntero esta dentro
			mouseOver =  rectangle.contains(new Point((int)mouseX,(int)mouseY));
			//Si acaba de hacer mouse over por primera vez ponemos un sonido
			if(estadoPrevio != mouseOver && mouseOver == true){
				GestorSonido.getGestorSonido().playEfectoSonido(GestorSonido.MOUSE_OVER);
			}
		}			
	}
	
	//Establece el rentangulo que contiene el texto
	private void setRectangle(Graphics g){
		FontMetrics metrics = g.getFontMetrics();
		int ancho  = metrics.stringWidth(texto);
		int alto = metrics.getHeight();
		
		int x = (int)(posX - (ancho/2));		
		int y = (int)(posY - (alto/2));
		
		rectangle = new Rectangle(x, y,ancho,alto);		
	}
	
	//Devuelve el punto para dibujar el texto
	private Point getPosicionADibujar(){		
		int x = (int)rectangle.getX();	
		int y = (int)(rectangle.getY()+rectangle.getHeight());
		
		return new Point(x,y);
	}
	
	//Comprueba si el boton ha sido pulsado	
	protected void comprobarInteracion() {
		try {
			//Si el usuario ha presionado el click izquierdo
			if(ControladorRaton.getControladorRaton().getClickIzquierdoPresionado()){
				//Obtenemos la posicin del puntero
				Point p = ControladorRaton.getControladorRaton().getPosicion();
				//Comprobamos si esta dentro			
				boolean b = rectangle.contains(p);
				if(b){
					//Avisamos al listener
					GestorSonido.getGestorSonido().playEfectoSonido(GestorSonido.CLICK);
					avisarListener();
				}				
			}
		} catch (Exception e) {
			e.printStackTrace();
		}	
		
	}	
	
	public void update(){
		super.update();
		//Comprobamos si el raton esta encima del boton
		comprobarMouseOver();	
		
	}	
	
	//Dibujar grficos
	public void draw(Graphics g){
		//Comprobamos si esta el mouse encima y ponemos el estilo que corresponda		
		if(mouseOver){			
			setEstiloMouseOver(g);
		}else
		{
			setEstiloNormal(g);
		}
		//Actualizamos el rectangulo dependiendo el estilo ya que uno ocupar mas que otro
		setRectangle(g);
		//Dibujamos el texto
		Point p = getPosicionADibujar();		
		g.drawString(texto, (int)p.getX(), (int)p.getY());
				
	}	
}
