package MotorGrafico.Utilidades;

import java.io.ByteArrayInputStream;
import java.io.StringWriter;
import java.util.HashMap;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;


public class GestorXML {

	private static GestorXML miGestorXML;
	
	private static final String PATH_CONFIGURACION = "configuracion.xml";	
	private Element documento;
	
	
	private GestorXML(){
		cargarXML();
	}
	
	public static GestorXML getGestorXML(){
		if(miGestorXML == null){
			miGestorXML = new GestorXML();
		}
		return miGestorXML;
	}
	
	
	private void cargarXML(){	
		
		try {
			//Cargamos el fichero XML desde el path predefinido
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			DocumentBuilder db = dbf.newDocumentBuilder();
			Document document = db.parse(getClass().getClassLoader().getResourceAsStream(PATH_CONFIGURACION));
			
			
			//Leemos la primera etiqueta con su contenido y lo almacenamos en una variable de clase 
			Element element = document.getDocumentElement();
			this.documento = element;
			
		} catch (Exception e) {
			e.printStackTrace();
		}		
	}
	
	//Devuelve el valor de una etiqueta del fichero XML de configuracion almacenado localmente
	public String obtenerValorConfiguracion(String tipo,String nombre){
			
		//Comprobamos que el fichero XML exista
		if(documento == null){
			cargarXML();
		}		
		//Nos colocamos en la etiqueta dependiendo el tipo
		Element e = (Element)documento.getElementsByTagName(tipo).item(0);
		//Accedemos a la etiqueta dentro del tipo
		String s = e.getElementsByTagName(nombre).item(0).getTextContent();	
			
		return s;
			
	}
	
	//Dado una lista de pares (etiqueta , valor) devuelve un string que representa un XML
	public String obtenerXML(HashMap<String,String> lista){
		try {			
			//Creamos el documento
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			DocumentBuilder db = dbf.newDocumentBuilder();
			Document document = db.newDocument();			
			
			//Creamos la etiqueta raiz
			Element eRaiz = document.createElement("datos");
			document.appendChild(eRaiz);
			
			//Recorremos la lista pares
			for (HashMap.Entry<String, String> x : lista.entrySet()) {
			    //Obtenemos la etiqueta y el valor
				String etiqueta = x.getKey();
			    String valor = x.getValue();
			    
			    //Creamos la etiqueta
			    Element e = document.createElement(etiqueta);
			    //Ponemos el valor
			    e.appendChild(document.createTextNode(valor));
			    //Lo aadimos al documento como hijo de la raiz
				eRaiz.appendChild(e);				
			}
			
			//Convertimos a string
			TransformerFactory tf = TransformerFactory.newInstance();
			Transformer t = tf.newTransformer();
			t.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
			StringWriter sw = new StringWriter();
			t.transform(new DOMSource(document), new StreamResult(sw));
			
			String resultado = sw.getBuffer().toString().replaceAll("\n|\r", "");			
			
			return resultado;
			
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		
		return null;		
	}
	
	//Dado una etiqueta y un XML devuelve el valor de esa etiqueta dentro del XML
	public String obtenerValorXML(String ficheroXML,String etiqueta){
		try {
			//Convertimos el fichero
			Element e = convertirXML(ficheroXML);
			//Obtenemos el valor de la etiqueta
			String valor = e.getElementsByTagName(etiqueta).item(0).getTextContent();
			
			return valor;			
			
		} catch (Exception e) {
			e.printStackTrace();
		}		
		
		return null;		
	}
	
	//Convierte de string a un objeto Element para leer el XML facilmente
	private Element convertirXML(String ficheroXML) throws Exception{
		
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		Document document = db.parse(new ByteArrayInputStream(ficheroXML.getBytes("UTF-8")));
		Element e = document.getDocumentElement();
		
		return e;
	}	
}
