package Pong.ControladoresEntrada;

import java.awt.event.KeyEvent;

public class ControladorTeclado {

	private static ControladorTeclado miControladorTeclado;	
	
	private boolean[] listaTeclasActuales;
	private boolean[] listaTeclasPrevias;
	
	//Cdigos de las teclas a usar
	public static final int ABAJO = KeyEvent.VK_S;
	public static final int ARRIBA = KeyEvent.VK_W;
	public static final int P = KeyEvent.VK_P;
	public static final int ESC = KeyEvent.VK_ESCAPE;
	
	private static final int NUM_TECLAS = 100;
	
	//Constructor
	private ControladorTeclado() {
		inicializarTeclado();
	}
	
	private void inicializarTeclado(){
		this.listaTeclasActuales = new boolean[NUM_TECLAS];
		this.listaTeclasPrevias = new boolean[NUM_TECLAS];
		//Inicializamos cada tecla de la lista a false ya que ninguna tecla ha sido pulsada
		for (int i = 0; i < listaTeclasActuales.length; i++) {
			listaTeclasActuales[i] = false;
			listaTeclasPrevias[i] = false;
		}
	}
	//Getter
	public static ControladorTeclado getControladorTeclado(){
		if( miControladorTeclado == null){
			 miControladorTeclado = new ControladorTeclado();
		}
		return miControladorTeclado;
	}
		
	//Tecla pulsada
	public void teclaPresionada(int codigo){
		if(codigo < NUM_TECLAS){
			listaTeclasActuales[codigo] = true;
		}
		
	}
	//Tecla liberada
	public void teclaLiberada(int codigo){
		if(codigo < NUM_TECLAS){
			listaTeclasActuales[codigo] = false;
		}		
	}
	//Devuelve si una tecla esta pulsada
	public boolean estaPulsada(int codigo){
		if(codigo < NUM_TECLAS){
			return listaTeclasActuales[codigo];
		}
		return false;
	}
	//Devueve si una tecla ha sido pulsada y liberada a continuacin
	public boolean haSidoPulsada(int codigo){
		return listaTeclasActuales[codigo] && !listaTeclasPrevias[codigo];
	}
	//Actualiza la lista de teclas previas
	public void actualizarTeclado(){
		for (int i = 0; i < listaTeclasPrevias.length; i++) {			
			listaTeclasPrevias[i] = listaTeclasActuales[i];
		}
	}
}
