package Pong.Estados.Partida;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.util.ArrayList;
import java.util.HashMap;


import Pong.Estados.IState;
import Pong.Estados.StateMachine;
import Pong.GestorJuego.GestorPrincipal;
import Pong.InterfazUsuario.BotonInterfaz;
import Pong.InterfazUsuario.IUIListener;
import Pong.InterfazUsuario.IconoSonido;
import Pong.InterfazUsuario.Imagen;
import Pong.InterfazUsuario.Texto;
import Pong.InterfazUsuario.UIObject;
import Pong.InterfazUsuario.UIObjectInteractive;
import Pong.Utilidades.GestorUsuarios;
import Pong.Utilidades.GestorXML;

public class EstadoBuscandoPartida implements IState,IUIListener {

	//Componentes para pintar la pantalla
	private ArrayList<UIObject> listaObjetosInterfaz;
	
	private boolean identificado;
	
	//Tiempo entre solicitudes
	private long ultimaSolicitud;
	private long tiempoEntreSolicitudes;
	
	public EstadoBuscandoPartida() {
		tiempoEntreSolicitudes = 1000;
		crearObjetosInterfaz();	
	}
	
	public void onSet() {
		identificado = false;
		ultimaSolicitud = 0;
	}	
	public void onChange() {}
	
	private void crearObjetosInterfaz() {
		listaObjetosInterfaz = new ArrayList<UIObject>();
		//Creamos el fondo
		Dimension d = GestorPrincipal.getGestorPrincipal().getDimensionPantalla();			
		Imagen fondo = new Imagen(0, 0, (int)d.getWidth(),(int)d.getHeight(), "/Imagenes/background.png");
		listaObjetosInterfaz.add(fondo);
		
		//Creamos el texto de buscando partida
		Texto t = new Texto(450, 225, "SEARCHING FOR MATCH...", Texto.CENTER_ALIGNMENT, true, Color.RED, 60);
		listaObjetosInterfaz.add(t);
		
		//Creamos el icono de sonido
		IconoSonido is = new IconoSonido(860, 410, "sonido", 30, 30);		
		listaObjetosInterfaz.add(is);
		//Botones
		BotonInterfaz bi = new BotonInterfaz(450, 400, "atras", "CANCEL",30);
		bi.addListener(this);
		listaObjetosInterfaz.add(bi);		
	}

	//Actualizamos
	public void update() {
		for (UIObject u1 : listaObjetosInterfaz) {
			u1.update();
		}
		enviarIdentificacion();
		solicitarDatos();
	}	
	
	private void solicitarDatos(){
		//Comprobamos si podemos enviar
		boolean tiempoTranscurrido = (System.currentTimeMillis() - ultimaSolicitud) > tiempoEntreSolicitudes;
		if(tiempoTranscurrido){
			//Creamos el fichero XML con la informacin para el servidor
			HashMap<String,String> lista = new HashMap<String,String>();
			lista.put("tipo", "obtenerDatos");
			GestorXML gestor = GestorXML.getGestorXML();
			String mensaje = gestor.obtenerXML(lista);
			
			//Enviamos al servidor
			GestorPrincipal gestorPrincipal = GestorPrincipal.getGestorPrincipal();
			gestorPrincipal.sendToServer(mensaje);			
			ultimaSolicitud = System.currentTimeMillis();
		}				
	}
	
	private void enviarIdentificacion(){
		//Identificarse servidor si no nos hemos identificado ya
		if(!identificado){
			identificado = true;
			//Creamos el fichero XML con la informacin para el servidor
			HashMap<String,String> lista = new HashMap<String,String>();
			lista.put("tipo", "envioDatos");
			String idUsuario = GestorUsuarios.getGestorUsuarios().getUsuarioActual().getIdentificador();
			lista.put("identificador",idUsuario);
			GestorXML gestor = GestorXML.getGestorXML();
			String mensaje = gestor.obtenerXML(lista);
							
			//Enviamos al servidor
			GestorPrincipal gestorPrincipal = GestorPrincipal.getGestorPrincipal();
			gestorPrincipal.sendToServer(mensaje);	
		}
		
	}

	//Dibujamos pantalla
	public void render(Graphics g) {
		for (UIObject u1 : listaObjetosInterfaz) {
			u1.draw(g);
		}		
	}	
	//Finalizar conexion servidor 
	public void interaccion(UIObjectInteractive object) {
		String id = object.getIdentificador();
		if(id.contentEquals("atras")){	
			GestorPrincipal gestorPrincipal = GestorPrincipal.getGestorPrincipal();			
			gestorPrincipal.finalizarConexion();			
		}		
	}
	
	public void receiveFromServer(String msg) {
		//Comprobamos el estado que envia el servidor
		GestorXML gestorXML = GestorXML.getGestorXML();
		String estadoActual = gestorXML.obtenerValorXML(msg, "estadoActual");		
		//Si hemos encontrado partida cambiamos el estado
		if(!estadoActual.contentEquals("estadoBuscandoPartida")){			
			GestorPrincipal.getGestorPrincipal().cambiarEstado(StateMachine.INICIANDO_PARTIDA);
		}
		
	}
}
