package Pong.Estados.Partida;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.util.ArrayList;
import java.util.HashMap;

import Pong.Estados.IState;
import Pong.GestorJuego.GestorPrincipal;
import Pong.InterfazUsuario.BotonInterfaz;
import Pong.InterfazUsuario.IUIListener;
import Pong.InterfazUsuario.IconoSonido;
import Pong.InterfazUsuario.Imagen;
import Pong.InterfazUsuario.Texto;
import Pong.InterfazUsuario.UIObject;
import Pong.InterfazUsuario.UIObjectInteractive;
import Pong.Sonido.GestorSonido;
import Pong.Utilidades.GestorXML;


public class EstadoFinPartida implements IState, IUIListener{

	//Componentes para pintar la pantalla
	private ArrayList<UIObject> listaObjetosInterfaz;	
	
	private Texto textoVictoria;
	private Texto textoCausa;
	
	public EstadoFinPartida() {
		crearObjetosInterfaz();
	}	
	
	private void crearObjetosInterfaz() {
		listaObjetosInterfaz = new ArrayList<UIObject>();
		//Creamos el fondo
		Dimension d = GestorPrincipal.getGestorPrincipal().getDimensionPantalla();			
		Imagen fondo = new Imagen(0, 0, (int)d.getWidth(),(int)d.getHeight(), "/Imagenes/background.png");
		listaObjetosInterfaz.add(fondo);
		
		//Creamos el texto
		Texto t = new Texto(450, 225, "GAME OVER", Texto.CENTER_ALIGNMENT, false, Color.white, 60);
		listaObjetosInterfaz.add(t);
		
		//Creamos el icono de sonido
		IconoSonido is = new IconoSonido(860, 410, "sonido", 30, 30);
		is.addListener(this);
		listaObjetosInterfaz.add(is);
		
		//Creamos el texto de ganador o perdedor y el de causa
		textoVictoria = new Texto(450, 325, "", Texto.CENTER_ALIGNMENT, true, Color.RED, 60);
		listaObjetosInterfaz.add(textoVictoria);	
		
		textoCausa = new Texto(450, 400, "", Texto.CENTER_ALIGNMENT, false, Color.WHITE, 60);
		listaObjetosInterfaz.add(textoCausa);
		
		//Botones
		BotonInterfaz bi = new BotonInterfaz(450, 400, "menu", "MAIN MENU",30);
		bi.addListener(this);
		listaObjetosInterfaz.add(bi);
	}

	public void onSet() {
		//Ponemos la musica
		GestorSonido.getGestorSonido().ponerMusica(GestorSonido.MUSICA_GAME_OVER);
		//Obtenemos los datos una unica vez
		solicitarDatos();
	}	
	public void onChange() {
		//Paramos musica		
		GestorSonido.getGestorSonido().detenerMusica();
	}
	
	private void solicitarDatos(){
		//Creamos el fichero XML con la informacin para el servidor
		HashMap<String,String> lista = new HashMap<String,String>();
		lista.put("tipo", "obtenerDatos");
		GestorXML gestor = GestorXML.getGestorXML();
		String mensaje = gestor.obtenerXML(lista);
		
		//Enviamos al servidor
		GestorPrincipal gestorPrincipal = GestorPrincipal.getGestorPrincipal();
		gestorPrincipal.sendToServer(mensaje);
	}
	
	public void receiveFromServer(String msg) {		
		GestorXML gestor = GestorXML.getGestorXML();
		//Obtenemos si ha ganado o no y la causa
		String valorGanador = gestor.obtenerValorXML(msg, "esGanador");
		boolean esGanador = Boolean.parseBoolean(valorGanador);
		if(esGanador){
			textoVictoria.setTexto("YOU WIN");
		}else{
			textoVictoria.setTexto("YOU LOSE");
		}
		//Creamos el texto de la causa si es por desconexin
		String causa = gestor.obtenerValorXML(msg, "causa");
		if(causa.contentEquals("desconexion")){
			textoCausa.setTexto("OPPONENT DISCONNECTED");
		}		
	}	
	
	//Actualizacin
	public void update() {
		for (UIObject u1 : listaObjetosInterfaz) {
			u1.update();
		}		
	}
	//Dibujo
	public void render(Graphics g) {
		for (UIObject u1 : listaObjetosInterfaz) {
			u1.draw(g);
		}		
	}
	
	public void interaccion(UIObjectInteractive object) {
		String id = object.getIdentificador();
		if(id.contentEquals("menu")){
			//Si hace click en menu finalizamos la conexion
			GestorPrincipal gestorPrincipal = GestorPrincipal.getGestorPrincipal();			
			gestorPrincipal.finalizarConexion();	
		}else if(id.contentEquals("sonido")){
			GestorSonido.getGestorSonido().ponerMusica(GestorSonido.MUSICA_GAME_OVER);
		}		
		
	}

}
