package Pong.Estados.Partida;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.util.ArrayList;
import java.util.HashMap;

import Pong.ControladoresEntrada.ControladorTeclado;
import Pong.Estados.IState;
import Pong.Estados.StateMachine;
import Pong.GestorJuego.GestorPrincipal;
import Pong.InterfazUsuario.IUIListener;
import Pong.InterfazUsuario.IconoSonido;
import Pong.InterfazUsuario.Imagen;
import Pong.InterfazUsuario.Texto;
import Pong.InterfazUsuario.UIObject;
import Pong.InterfazUsuario.UIObjectInteractive;
import Pong.Utilidades.GestorXML;

public class EstadoIniciandoPartida implements IState,IUIListener{

	//Componentes para pintar la pantalla
	private ArrayList<UIObject> listaObjetosInterfaz;	
	
	private Texto textoCuentaAtras;
	
	//Tiempo entre solicitudes
	private long ultimaSolicitud;
	private long tiempoEntreSolicitudes;
	
	public EstadoIniciandoPartida() {
		tiempoEntreSolicitudes = 300;		
		crearObjetosInterfaz();	
	}
	
	public void onSet() {
		ultimaSolicitud = 0;
	}	
	public void onChange() {}
	
	private void crearObjetosInterfaz() {
		listaObjetosInterfaz = new ArrayList<UIObject>();
		//Creamos el fondo
		Dimension d = GestorPrincipal.getGestorPrincipal().getDimensionPantalla();			
		Imagen fondo = new Imagen(0, 0, (int)d.getWidth(),(int)d.getHeight(), "/Imagenes/background.png");
		listaObjetosInterfaz.add(fondo);
		
		//Creamos el icono de sonido
		IconoSonido is = new IconoSonido(860, 410, "sonido", 30, 30);
		is.addListener(this);
		listaObjetosInterfaz.add(is);
		
		//Creamos el texto
		Texto t = new Texto(450, 225, "STARTING MATCH", Texto.CENTER_ALIGNMENT, false, Color.white, 60);
		listaObjetosInterfaz.add(t);		
		
		textoCuentaAtras = new Texto(450,325,"",Texto.CENTER_ALIGNMENT,false,Color.red,60);
		listaObjetosInterfaz.add(textoCuentaAtras);		
	}

	//Actualizamos
	public void update() {
		checkInputs();
		for (UIObject u1 : listaObjetosInterfaz) {
			u1.update();
		}
		solicitarDatos();
	}
	
	//Comprueba la entrada datos por teclado
	private void checkInputs(){
		ControladorTeclado controladorTeclado = ControladorTeclado.getControladorTeclado();
		if(controladorTeclado.haSidoPulsada(ControladorTeclado.ESC)){
			//Si pulsa ESC salimos de la partida
			GestorPrincipal gestorPrincipal = GestorPrincipal.getGestorPrincipal();
			gestorPrincipal.finalizarConexion();
		}
	}

	//Dibujamos pantalla
	public void render(Graphics g) {
		for (UIObject u1 : listaObjetosInterfaz) {
			u1.draw(g);
		}		
	}
	
	public void receiveFromServer(String msg) {
		GestorXML gestor = GestorXML.getGestorXML();
		String estado = gestor.obtenerValorXML(msg, "estadoActual");
		//Si no ha cambiado el estado actualizamos el tiempo restante
		if(estado.contentEquals("estadoIniciandoPartida")){
			String tiempoRestante = gestor.obtenerValorXML(msg, "tiempoRestante");
			this.textoCuentaAtras.setTexto(tiempoRestante);
		}else if(estado.contentEquals("estadoJugando")){
			//Si ha cambiado a jugando ponemos el estado jugando
			GestorPrincipal gestorPrincipal = GestorPrincipal.getGestorPrincipal();
			gestorPrincipal.cambiarEstado(StateMachine.JUGANDO);
		}else if(estado.contentEquals("estadoFinPartida")){
			//Si ha cambiado a fin partida ponemos el estado finPartida
			GestorPrincipal gestorPrincipal = GestorPrincipal.getGestorPrincipal();
			gestorPrincipal.cambiarEstado(StateMachine.FIN_PARTIDA);
		}
	}
	
	private void solicitarDatos(){
		//Comprobamos si podemos enviar
		boolean tiempoTranscurrido = (System.currentTimeMillis() - ultimaSolicitud) > tiempoEntreSolicitudes;
		if(tiempoTranscurrido){
			//Creamos el fichero XML con la informacin para el servidor
			HashMap<String,String> lista = new HashMap<String,String>();
			lista.put("tipo", "obtenerDatos");
			GestorXML gestor = GestorXML.getGestorXML();
			String mensaje = gestor.obtenerXML(lista);
			
			//Enviamos al servidor
			GestorPrincipal gestorPrincipal = GestorPrincipal.getGestorPrincipal();
			gestorPrincipal.sendToServer(mensaje);
			
			ultimaSolicitud = System.currentTimeMillis();
		}	
	}	
	public void interaccion(UIObjectInteractive object) {}
}
