package Pong.Estados.Partida;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.util.ArrayList;
import java.util.HashMap;

import Pong.ControladoresEntrada.ControladorTeclado;
import Pong.Estados.IState;
import Pong.Estados.StateMachine;
import Pong.GameplayObjects.Bola;
import Pong.GameplayObjects.Paddle;
import Pong.GestorJuego.GestorPrincipal;
import Pong.InterfazUsuario.IUIListener;
import Pong.InterfazUsuario.IconoSonido;
import Pong.InterfazUsuario.Imagen;
import Pong.InterfazUsuario.Texto;
import Pong.InterfazUsuario.UIObject;
import Pong.InterfazUsuario.UIObjectInteractive;

import Pong.Utilidades.GestorXML;

public class EstadoJugando implements IState,IUIListener{
	
	//Componentes para pintar la pantalla
	private ArrayList<UIObject> listaObjetosInterfaz;	
	
	private Texto puntuacionJugador;
	private Texto puntuacionOponente;
	
	//Tiempo entre solicitudes
	private long ultimaSolicitud;
	private long tiempoEntreSolicitudes;
	
	//Objetos del juego (paletas y bola)
	private Paddle paletaJugador;
	private Paddle paletaOponente;
	private Bola bola;	
	
	
	public EstadoJugando() {
		tiempoEntreSolicitudes = 1000/64;
		crearObjetosInterfaz();
		bola = new Bola();
		crearPaletas();
	}
	
	private void crearPaletas() {
		//Cargamos los valores de la configuracin
		GestorXML gestorXML = GestorXML.getGestorXML();
		String s = gestorXML.obtenerValorConfiguracion("jugador", "posYInicial");
		double posY = Double.parseDouble(s);
		s = gestorXML.obtenerValorConfiguracion("jugador", "posXIzquierda");
		double xIzquierda = Double.parseDouble(s);
		s = gestorXML.obtenerValorConfiguracion("jugador", "posXDerecha");
		double xDerecha = Double.parseDouble(s);
		paletaJugador = new Paddle(xIzquierda, posY);
		paletaOponente = new Paddle(xDerecha, posY);		
	}

	public void onSet() {
		ultimaSolicitud = 0;
	}	
	public void onChange() {}
	
	private void crearObjetosInterfaz() {
		listaObjetosInterfaz = new ArrayList<UIObject>();
		//Creamos el fondo
		Dimension d = GestorPrincipal.getGestorPrincipal().getDimensionPantalla();			
		Imagen fondo = new Imagen(0, 0, (int)d.getWidth(),(int)d.getHeight(), "/Imagenes/background.png");
		listaObjetosInterfaz.add(fondo);
		//Creamos el icono de sonido
		IconoSonido is = new IconoSonido(860, 410, "sonido", 30, 30);
		is.addListener(this);
		listaObjetosInterfaz.add(is);
		
		//Creamos los textos para poner la info		
		puntuacionJugador = new Texto(200, 180, "", Texto.CENTER_ALIGNMENT, false, Color.WHITE, 100);
		listaObjetosInterfaz.add(puntuacionJugador);
		puntuacionOponente = new Texto(700, 180, "", Texto.CENTER_ALIGNMENT, false, Color.WHITE, 100);
		listaObjetosInterfaz.add(puntuacionOponente);		
	}

	//Actualizacin
	public void update() {
		checkInputs();		
		paletaJugador.update();
		for (UIObject u1 : listaObjetosInterfaz) {
			u1.update();
		}
		solicitarYEnviarDatos();
	}	

	//Comprueba la entrada datos por teclado
	private void checkInputs(){
		ControladorTeclado controladorTeclado = ControladorTeclado.getControladorTeclado();
		if(controladorTeclado.haSidoPulsada(ControladorTeclado.P)){			
			//Si el usuario presiona tecla P solicitamos una pausa
			//Creamos el fichero XML enviar datos
			HashMap<String,String> lista = new HashMap<String,String>();
			lista.put("tipo", "envioFuncion");
			lista.put("idFuncion", "pausar");
			GestorXML gestor = GestorXML.getGestorXML();
			String mensaje = gestor.obtenerXML(lista);
			
			//Enviamos al servidor
			GestorPrincipal gestorPrincipal = GestorPrincipal.getGestorPrincipal();
			gestorPrincipal.sendToServer(mensaje);			
		}else if(controladorTeclado.haSidoPulsada(ControladorTeclado.ESC)){
			//Si pulsa ESC salimos de la partida
			GestorPrincipal gestorPrincipal = GestorPrincipal.getGestorPrincipal();
			gestorPrincipal.finalizarConexion();
		}
	}
	
	//Dibujo
	public void render(Graphics g) {		
		for (UIObject u1 : listaObjetosInterfaz) {
			u1.draw(g);
		}
		dibujarLineaTablero(g);
		bola.draw(g);
		paletaJugador.draw(g);
		paletaOponente.draw(g);
	}
	//Dibuja la linea divisoria del campo
	public void dibujarLineaTablero(Graphics g){
		int numLineas = 20;
		int ancho = 5;
		int alto = 16;
		int x = 450+(ancho/2);
		int y = 5;
		int aumentoY = 22;		
		g.setColor(Color.white);
		for (int i = 0; i < numLineas; i++) {
			g.fillRect(x, y, ancho, alto);
			y +=aumentoY;
		}
	}
	
	private void solicitarYEnviarDatos(){
		//Comprobamos si podemos enviar
		boolean tiempoTranscurrido = (System.currentTimeMillis() - ultimaSolicitud) > tiempoEntreSolicitudes;
		if(tiempoTranscurrido){
			//Creamos el fichero XML para solicitar datos
			HashMap<String,String> lista = new HashMap<String,String>();
			lista.put("tipo", "obtenerDatos");
			GestorXML gestor = GestorXML.getGestorXML();
			String mensaje = gestor.obtenerXML(lista);
			
			//Enviamos al servidor
			GestorPrincipal gestorPrincipal = GestorPrincipal.getGestorPrincipal();
			gestorPrincipal.sendToServer(mensaje);
			
			//Creamos el fichero XML con el envio datos
			lista = new HashMap<String,String>();
			lista.put("tipo", "envioDatos");
			double posY = paletaJugador.getPosicionY();
			lista.put("posY",String.valueOf(posY));
			mensaje = gestor.obtenerXML(lista);
			
			//Enviamos al servidor			
			gestorPrincipal.sendToServer(mensaje);
			
			ultimaSolicitud = System.currentTimeMillis();
			
		}	
	}
	
	public void receiveFromServer(String msg) {
		//Comprobamos latencia
		long transcurrido = System.currentTimeMillis() - ultimaSolicitud;
		System.out.println(transcurrido);
		//Obtenemos el estado en que estamos
		GestorXML gestor = GestorXML.getGestorXML();
		String estado = gestor.obtenerValorXML(msg, "estadoActual");		
		if(estado.contentEquals("estadoJugando")){
			//Si seguimos jugando obtenemos los demas datos
			String valor = gestor.obtenerValorXML(msg, "posYOponente");
			paletaOponente.setPosicionY(Double.parseDouble(valor));
			
			valor = gestor.obtenerValorXML(msg, "bolaX");
			double bolaX = Double.parseDouble(valor);
			valor = gestor.obtenerValorXML(msg, "bolaY");
			double bolaY = Double.parseDouble(valor);
			bola.setPos(bolaX, bolaY);
			valor = gestor.obtenerValorXML(msg, "direccionBolaX");
			double dX = Double.parseDouble(valor);
			bola.setDireccionX(dX);
			
			valor = gestor.obtenerValorXML(msg, "puntuacionJugador");
			puntuacionJugador.setTexto(valor);
			valor = gestor.obtenerValorXML(msg, "puntuacionOponente");
			puntuacionOponente.setTexto(valor);	
			
			
		}else if(estado.contentEquals("estadoPausa")){
			//Si se ha establecido una pausa cambiamos el estado
			GestorPrincipal gestorPrincipal = GestorPrincipal.getGestorPrincipal();
			gestorPrincipal.cambiarEstado(StateMachine.PAUSA);
		}else if(estado.contentEquals("estadoFinPartida")){
			GestorPrincipal gestorPrincipal = GestorPrincipal.getGestorPrincipal();
			gestorPrincipal.cambiarEstado(StateMachine.FIN_PARTIDA);
		}
		
	}
	
	public void interaccion(UIObjectInteractive object) {}	

}
