package Pong.Estados.Partida;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.util.ArrayList;
import java.util.HashMap;

import Pong.ControladoresEntrada.ControladorTeclado;
import Pong.Estados.IState;
import Pong.Estados.StateMachine;
import Pong.GestorJuego.GestorPrincipal;
import Pong.InterfazUsuario.IUIListener;
import Pong.InterfazUsuario.IconoSonido;
import Pong.InterfazUsuario.Imagen;
import Pong.InterfazUsuario.Texto;
import Pong.InterfazUsuario.UIObject;
import Pong.InterfazUsuario.UIObjectInteractive;
import Pong.Utilidades.GestorXML;

public class EstadoPausa implements IState,IUIListener{

	//Componentes para pintar la pantalla
	private ArrayList<UIObject> listaObjetosInterfaz;
	
	private Texto textoCuentaAtras;
	private Texto textoPausa;
	
	//Tiempo entre solicitudes
	private long ultimaSolicitud;
	private long tiempoEntreSolicitudes;
	
	public EstadoPausa() {
		tiempoEntreSolicitudes = 400;
		crearObjetosInterfaz();
	}	
	
	private void crearObjetosInterfaz() {
		listaObjetosInterfaz = new ArrayList<UIObject>();
		//Creamos el fondo
		Dimension d = GestorPrincipal.getGestorPrincipal().getDimensionPantalla();			
		Imagen fondo = new Imagen(0, 0, (int)d.getWidth(),(int)d.getHeight(), "/Imagenes/background.png");
		listaObjetosInterfaz.add(fondo);
		
		//Creamos el icono de sonido
		IconoSonido is = new IconoSonido(860, 410, "sonido", 30, 30);
		is.addListener(this);
		listaObjetosInterfaz.add(is);
		
		//Creamos el texto
		textoPausa = new Texto(450, 225, "", Texto.CENTER_ALIGNMENT, false, Color.white, 60);
		listaObjetosInterfaz.add(textoPausa);
		
		textoCuentaAtras = new Texto(450,325,"",Texto.CENTER_ALIGNMENT,false,Color.red,60);
		listaObjetosInterfaz.add(textoCuentaAtras);		
		
	}

	public void onSet() {		
		ultimaSolicitud = 0;
	}	
	public void onChange() {}
	
	public void update() {
		checkInputs();
		for (UIObject u1 : listaObjetosInterfaz) {
			u1.update();
		}
		solicitarDatos();		
	}
	
	//Comprueba la entrada datos por teclado
	private void checkInputs(){
		ControladorTeclado controladorTeclado = ControladorTeclado.getControladorTeclado();
		if(controladorTeclado.haSidoPulsada(ControladorTeclado.P)){
			//Si el usuario presiona tecla P solicitamos reanudar partida
			//Creamos el fichero XML enviar datos
			HashMap<String,String> lista = new HashMap<String,String>();
			lista.put("tipo", "envioFuncion");
			lista.put("idFuncion", "reanudar");
			GestorXML gestor = GestorXML.getGestorXML();
			String mensaje = gestor.obtenerXML(lista);
			
			//Enviamos al servidor
			GestorPrincipal gestorPrincipal = GestorPrincipal.getGestorPrincipal();
			gestorPrincipal.sendToServer(mensaje);			
		}else if(controladorTeclado.haSidoPulsada(ControladorTeclado.ESC)){
			//Si pulsa ESC salimos de la partida
			GestorPrincipal gestorPrincipal = GestorPrincipal.getGestorPrincipal();
			gestorPrincipal.finalizarConexion();
		}
	}
	
	private void solicitarDatos() {
		//Comprobamos si podemos enviar
		boolean tiempoTranscurrido = (System.currentTimeMillis() - ultimaSolicitud) > tiempoEntreSolicitudes;
		if(tiempoTranscurrido){
			//Creamos el fichero XML con la informacin para el servidor
			HashMap<String,String> lista = new HashMap<String,String>();
			lista.put("tipo", "obtenerDatos");
			GestorXML gestor = GestorXML.getGestorXML();
			String mensaje = gestor.obtenerXML(lista);
			
			//Enviamos al servidor
			GestorPrincipal gestorPrincipal = GestorPrincipal.getGestorPrincipal();
			gestorPrincipal.sendToServer(mensaje);
			
			ultimaSolicitud = System.currentTimeMillis();
		}	
		
	}
	
	//Respuesta servidor
	public void receiveFromServer(String msg) {
		GestorXML gestor = GestorXML.getGestorXML();
		String estado = gestor.obtenerValorXML(msg, "estadoActual");
		//Si no ha cambiado el estado actualizamos los valores
		if(estado.contentEquals("estadoPausa")){
			//Obtenemos el tiempo restante
			String tiempoRestante = gestor.obtenerValorXML(msg, "tiempoRestante");
			this.textoCuentaAtras.setTexto(tiempoRestante);
			//Obtenemos si estamos en reanudacion o en pausa
			String valorReanudando = gestor.obtenerValorXML(msg, "reanudandoPartida");
			boolean reanudandoPartida = Boolean.parseBoolean(valorReanudando);			
			if(reanudandoPartida){
				this.textoPausa.setTexto("RESUMING");
			}else{
				//Cambiamos el texto de acuerdo si este usuario ha pausado la partida o el oponente
				String valorSolicitante = gestor.obtenerValorXML(msg, "esSolicitante");
				boolean esSolicitante = Boolean.parseBoolean(valorSolicitante);	
				if(esSolicitante){
					this.textoPausa.setTexto("YOU PAUSED THE GAME");
				}else{
					this.textoPausa.setTexto("OPPONENT PAUSED THE GAME");
				}
			}
			
		}else if(estado.contentEquals("estadoJugando")){
			//Si se ha quitado la pausa ponemos estado jugando
			GestorPrincipal gestorPrincipal = GestorPrincipal.getGestorPrincipal();
			gestorPrincipal.cambiarEstado(StateMachine.JUGANDO);
		}else if(estado.contentEquals("estadoFinPartida")){
			GestorPrincipal gestorPrincipal = GestorPrincipal.getGestorPrincipal();
			gestorPrincipal.cambiarEstado(StateMachine.FIN_PARTIDA);
		}			
		
	}

	public void render(Graphics g) {
		for (UIObject u1 : listaObjetosInterfaz) {
			u1.draw(g);
		}			
	}

	
	public void interaccion(UIObjectInteractive object) {	}
	
	

	

}
