package Pong.Estados;

import java.awt.Graphics;

import Pong.Estados.Partida.EstadoBuscandoPartida;
import Pong.Estados.Partida.EstadoFinPartida;
import Pong.Estados.Partida.EstadoIniciandoPartida;
import Pong.Estados.Partida.EstadoJugando;
import Pong.Estados.Partida.EstadoPausa;

public class StateMachine {

	//Listado estados posibles
	public static final int MENU_INICIO = 0;
	public static final int INSTRUCCIONES = 1;
	public static final int VER_RANKING = 2;
	
	public static final int BUSCANDO_PARTIDA = 3;
	public static final int INICIANDO_PARTIDA = 4;
	public static final int JUGANDO = 5;
	public static final int PAUSA = 6;
	public static final int FIN_PARTIDA = 7;
	
	private static final int NUM_ESTADOS = 8;
	
	private IState[] listaEstados;
	
	private int estadoActual;	
	
	//Constructor
	public StateMachine() {
		inicilizar();		
	}	
	private void inicilizar() {
		crearEstados();
		//Ponemos el estado inicial
		setState(MENU_INICIO);		
	}		
	
	private void crearEstados(){
		listaEstados = new IState[NUM_ESTADOS];	
		listaEstados[MENU_INICIO] = new EstadoMenuInicio();
		listaEstados[INSTRUCCIONES] = new EstadoInstrucciones();
		listaEstados[VER_RANKING] = new EstadoVerRanking();
		listaEstados[BUSCANDO_PARTIDA] = new EstadoBuscandoPartida();
		listaEstados[INICIANDO_PARTIDA] = new EstadoIniciandoPartida();
		listaEstados[JUGANDO] = new EstadoJugando();
		listaEstados[PAUSA] = new EstadoPausa();
		listaEstados[FIN_PARTIDA] = new EstadoFinPartida();
	}
	
	public boolean esEstadoActual(int estado){
		return estado == estadoActual;
	}
		
	//Setter estado
	public void setState(int estado){
		if(listaEstados[estadoActual] != null){
			//Avisamos al estado actual del cambio
			listaEstados[estadoActual].onChange();
		}	
		//Avisamos al estado de que le vamos a establecer
		listaEstados[estado].onSet();
		estadoActual = estado;		
	}	
		
	//Actualiza el estado actual
	public void update(){
		if(listaEstados[estadoActual] != null){
			listaEstados[estadoActual].update();
		}			
	}
	//Renderiza el estado actual
	public void render(Graphics g){
		if(listaEstados[estadoActual] != null){
			listaEstados[estadoActual].render(g);
		}
	}	

	//Pasamos el mensaje del servidor al estado
	public void receiveFromServer(String msg) {
		if(listaEstados[estadoActual] != null){
			listaEstados[estadoActual].receiveFromServer(msg);
		}
	}	
	
}
