package Pong.GameplayObjects;

import java.awt.Color;
import java.awt.Graphics;

import Pong.ControladoresEntrada.ControladorTeclado;
import Pong.Utilidades.GestorXML;

public class Paddle {

	private int limiteArriba;
	private int limiteAbajo;
	
	private int ancho;
	private int alto;
	
	private double posX;
	private double posY;
	
	private int direccionY;
	
	private double speed;
	
	
	public Paddle(double x,double y) {
		this.posX = x;
		this.posY = y;
		this.direccionY = 0;
		inicializar();
	}
	
	private void inicializar() {
		//Cargamos las variables del fichero configuracin
		String s = GestorXML.getGestorXML().obtenerValorConfiguracion("tablero", "limiteArriba");
		this.limiteArriba = Integer.parseInt(s);
		
		s = GestorXML.getGestorXML().obtenerValorConfiguracion("tablero", "limiteAbajo");
		this.limiteAbajo = Integer.parseInt(s);
		
		s = GestorXML.getGestorXML().obtenerValorConfiguracion("jugador", "ancho");
		this.ancho = Integer.parseInt(s);
		
		s = GestorXML.getGestorXML().obtenerValorConfiguracion("jugador", "alto");
		this.alto = Integer.parseInt(s);	
		
		s = GestorXML.getGestorXML().obtenerValorConfiguracion("jugador", "velocidad");
		this.speed = Double.parseDouble(s);
	}
	
	public void setPosicionY(double y){		
		this.posY = y;
	}
	
	public double getPosicionY(){
		return posY;
	}
	
	//Actualizacin del paddle
	public void update(){
		checkInputs();
		move();
		comprobarLimites();
	}	
	//Mueve al jugador en la direccin apropiada
	private void move(){
		this.posY += speed*direccionY;
		
	}
	private void comprobarLimites(){
		//Comprobamos si estamos fuera de los limites
		if(direccionY > 0){
			if((posY + (alto/2)) > limiteAbajo){
				posY = limiteAbajo - (alto/2);
			}
		}else{
			if((posY - (alto/2)) < limiteArriba){
				posY = limiteArriba + (alto/2);
			}		
		}
	}
	//Comprueba las entradas por teclado
	private void checkInputs(){
		//Comprobamos si esta pulsada la tecla de arriba o abajo y  actualizamos la direccin
		ControladorTeclado ct = ControladorTeclado.getControladorTeclado();
		if(ct.estaPulsada(ControladorTeclado.ARRIBA)){
			direccionY = -1;
		}else if(ct.estaPulsada(ControladorTeclado.ABAJO)){
			direccionY = 1;
		}else{
			direccionY = 0;
		}
	}	

	//Dibuja el paddle
	public void draw(Graphics g){
		g.setColor(Color.white);
		int x = (int) (posX - (ancho/2));
		int y = (int) (posY - (alto/2));
		g.fillRect(x, y, ancho, alto);
	}	
}
