package Pong.GestorJuego;
import java.awt.Dimension;
import java.awt.Graphics;


import javax.swing.JFrame;
import javax.swing.JOptionPane;


import Pong.ConexionServidor.ControlCliente;

import Pong.Estados.StateMachine;
import Pong.Utilidades.GestorUsuarios;
import Pong.Utilidades.GestorXML;



public class GestorPrincipal {

	private static GestorPrincipal miGestorPingPong;
	
	private ControlCliente controlCliente;	
	private StateMachine stateMachine;
	private GameCanvas gameCanvas;
	
	private boolean iniciado;
	private boolean fin;	
	
	private GestorPrincipal() {
		iniciado = false;
		fin = false;
	}
	
	public static GestorPrincipal getGestorPrincipal(){
		if(miGestorPingPong == null){
			miGestorPingPong = new GestorPrincipal();
		}
		return miGestorPingPong;
	}
	
	//Iniciamos el juego
	public void iniciarJuego(int id,String usuario){
		//Comprobamos que el usuario sea correcto
		boolean usuarioCorrecto = GestorUsuarios.getGestorUsuarios().crearUsuarioActual(id, usuario);
		if(usuarioCorrecto){
			//Creamos la pantalla
			gameCanvas = new GameCanvas();
			//Creamos al maquina estados
			stateMachine = new StateMachine();		
			iniciado = true; 
		}else{
			error("User not valid.");
		}
				
	}
	
	//Devuelve la dimensin de la pantalla
	public Dimension getDimensionPantalla() {		
		String s = GestorXML.getGestorXML().obtenerValorConfiguracion("ventana", "ancho");
		int ancho = Integer.parseInt(s);
		
		s = GestorXML.getGestorXML().obtenerValorConfiguracion("ventana", "alto");
		int alto = Integer.parseInt(s);
		
		return new Dimension(ancho,alto);
	}
	
	//Cierra el juego y muestra el error
	public void error(String msg) {		
		if(!fin){			
			fin = true;						
			//Mostramos un panel de error
			JOptionPane.showMessageDialog(new JFrame(), msg, "ERROR",
			        JOptionPane.ERROR_MESSAGE);
			cerrarJuego();
			System.exit(1);
		}			
	}
	//Cierra el juego
	public void cerrarJuego(){
		if(!fin && iniciado){
			fin = true;
			finalizarConexion();
			gameCanvas.pararJuego();
		}
	}
	
	//Actualiza la maquina estados
	public void update(){
		if(stateMachine != null){
			stateMachine.update();
		}		
	}
	//Renderiza
	public void render(Graphics g){
		if(stateMachine != null){
			stateMachine.render(g);
		}	
	}
	
	//Cambia el estado de la maquina de estados
	public void cambiarEstado(int nuevoEstado){
		stateMachine.setState(nuevoEstado);
	}	
	
	//PARTE CONEXION	
	//Intentamos conectarnos al servidor
	public void iniciarConexion(){		
		if(controlCliente == null){
			controlCliente = new ControlCliente();
		}
		controlCliente.iniciarConexion();
	}	
	
	//Conexin no establecida
	public void conexionNoEstablecida(){
		//Mostramos un panel de error
		JOptionPane.showMessageDialog(new JFrame(), "Could not connect to server.", "ERROR",
		        JOptionPane.ERROR_MESSAGE);		
	}
	
	public synchronized void desconectadoServidor() {
		//Comprobamos que no este en el menu ya
		if(!stateMachine.esEstadoActual(StateMachine.MENU_INICIO)){
			//Mostramos un panel de error y volvemos al estado menu ininicial
			cambiarEstado(StateMachine.MENU_INICIO);
			JOptionPane.showMessageDialog(new JFrame(), "Disconnected from server.", "MESSAGE",
					 JOptionPane.ERROR_MESSAGE);	
		}			
	}
	//Cierra la conexion con el servidor
	public void finalizarConexion(){
		if(controlCliente != null){
			controlCliente.desconectarse();
		}		
	}
	
	public void receiveFromServer(String msg){
		if(stateMachine != null){
			stateMachine.receiveFromServer(msg);
		}	
	}
	
	public void sendToServer(String msg){
		controlCliente.sendToServer(msg);
	}	
}
