package Pong.InterfazUsuario;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Point;


public class Texto extends UIObject {

	public static final int LEFT_ALIGNMENT = 0;
	public static final int CENTER_ALIGNMENT = 1;
	
	private int alignment;
	private Color color;		
	private String texto;
	private int size;
	
	//Control del parpadeo
	private boolean parpadeo;
	private float alpha;
	private float cambioAlphaPorTick;
	private boolean direccionAbajo;
	
	public Texto(double posX, double posY,String texto,int alignment,boolean parpadeo,Color color,int size) {
		super(posX, posY);
		this.alignment = alignment;
		this.texto = texto;
		this.parpadeo = parpadeo;
		this.color = color;
		this.size = size;		
		this.alpha = 1.0f;
		this.direccionAbajo = true;
		this.cambioAlphaPorTick = 0.03f;
	}

	//Devuelve la posicin para dibujar el texto que depende de la alineacin
	private Point getPosicionADibujar(Graphics g){
		FontMetrics metrics = g.getFontMetrics();
		int ancho  = metrics.stringWidth(texto);
		int alto = metrics.getHeight();
		int x;
		int y;
		if(alignment == LEFT_ALIGNMENT){
			x = (int)posX;
			y = (int)(posY - (alto/2));
			return new Point(x,y);
		}else
		{			
			x = (int)(posX - (ancho/2));		
			y = (int)(posY - (alto/2));
			
			return new Point(x,y);
		}
	}
	
	public void setTexto(String texto){
		this.texto = texto;
	}
	
	//Establece el estilo grfico del texto
	private void setEstilo(Graphics g){
		Font font = new Font("Showcard Gothic",Font.PLAIN,size);
		g.setFont(font);
		//Si el texto parpadea usamos el canal alpha del color para simularlo
		if(parpadeo){
			
			Color c = new Color(color.getColorSpace(),color.getColorComponents(null),alpha);	
			g.setColor(c);
		}else
		{			
			g.setColor(color);
		}		
	}
	//Actualiza el alpha para simular el parpadeo
	private void actualizarAlpha(){
		
		if(direccionAbajo){
			alpha -=cambioAlphaPorTick;
			if(alpha < 0.0f){
				alpha = 0.0f;
				direccionAbajo = false;
			}
		}else
		{
			alpha +=cambioAlphaPorTick;
			if(alpha > 1.0f){
				alpha = 1.0f;
				direccionAbajo = true;
			}
		}
	}
	
	public void update() {
		if(parpadeo){
			actualizarAlpha();
		}
	}
	
	//Dibuja el texto en pantalla
	public void draw(Graphics g) {
		setEstilo(g);
		Point p = getPosicionADibujar(g);
		g.drawString(texto, p.x,p.y);
	}

	
	

}
