package GestorPrincipal;

import java.io.PrintWriter;
import java.util.HashMap;

import GestionClientes.Cliente;
import GestionClientes.DatosCliente;
import GestionClientes.ListaClientes;
import Partida.GestorPartidas;
import Servidor.ControladorServidor;

import SistemaEmparejamiento.GestorEmparejamientos;
import Utilidades.GestorXML;
import Vista.VistaServidor;

public class GestorPrincipal {

	private static GestorPrincipal miGestorPrincipal;
	
	private VistaServidor vistaServidor;
	
	private ListaClientes listaClientes;
	private ControladorServidor controladorServidor;
	private GestorPartidas gestorPartidas;
	private GestorEmparejamientos gestorEmparejamientos;
	
	
	//Constructor
	private GestorPrincipal(){
		inicializar();
	}
	
	private void inicializar(){
		//Iniciamos las variables
		this.listaClientes = new ListaClientes();
		this.controladorServidor = new ControladorServidor();
		this.gestorPartidas = new GestorPartidas();
		this.gestorEmparejamientos = new GestorEmparejamientos();
	}	
	
	public static GestorPrincipal getGestorPrincipal(){
		if(miGestorPrincipal == null){
			miGestorPrincipal = new GestorPrincipal();			
		}
		return miGestorPrincipal;
	}	
	
	public void mostrarMensaje(String msg){
		if(vistaServidor != null){
			vistaServidor.mostrarPorPantalla(msg);
		}		
	}
	
	//Inicia el servidor
	public void iniciarServidor(VistaServidor vs){
		this.vistaServidor = vs;
		this.controladorServidor.iniciarServidor();
	}
	
	//Aade un nuevo cliente
	public void clienteConectado(Cliente c){		
		listaClientes.introducirCliente(c);
	}
	
	public void clienteDesconectado(DatosCliente dc){
		//Obtenemos el cliente y cambiamoso su estado
		Cliente c = listaClientes.getCliente(dc.getStringDatos());		
		int estado = c.getEstado();
		if(estado == Cliente.ESTADO_BUSCANDO){			
			//Si esta buscando lo eliminamos del gestor emparejamiento
			gestorEmparejamientos.eliminarCliente(c);
		}		
		//Eliminamos de la lista general
		listaClientes.clienteDesconectado(dc);
	}
	
	//Crea una partida entre dos clientes
	public void crearPartida(Cliente c1,Cliente c2){
		gestorPartidas.crearPartida(c1, c2);
	}
	//Elimina una partida dado el identificador
	public void eliminarPartida(String key){
		gestorPartidas.eliminarPartida(key);
	}
	
	public void mensajeCliente(DatosCliente dc,String mensaje,PrintWriter out){		
		//Obtenemos el cliente
		Cliente c = listaClientes.getCliente(dc.getStringDatos());
		//Obtenemos el estado actual del cliente
		int estado = c.getEstado();
		if(estado == Cliente.ESTADO_CONECTADO){			
			//Comprobamos que el usuario se este identificando
			String tipo = GestorXML.getGestorXML().obtenerValorXML(mensaje, "tipo");
			if(tipo.contentEquals("envioDatos")){
				//Obtenemos el identificador que enva el cliente
				String id = GestorXML.getGestorXML().obtenerValorXML(mensaje, "identificador");				
				//Comprobamos que no exista ya el mismo usuario
				String keyCliente = c.getKey();
				boolean existe = listaClientes.existeUsuarioEnLista(id,keyCliente);
				if(!existe){
					c.identificarse(id);
					//Lo aadimos el sistema emparejamiento
					gestorEmparejamientos.addCliente(c);
				}else{
					//Si ya existe le hechamos del servidor
					controladorServidor.kickCliente(dc);
				}
				
			}
			
		}else if(estado == Cliente.ESTADO_BUSCANDO){
			//Comprobamos que el usuario solicite datos
			String tipo = GestorXML.getGestorXML().obtenerValorXML(mensaje, "tipo");
			if(tipo.contentEquals("obtenerDatos")){
				//Enviamos al usuario el estado actual
				//Creamos el fichero XML con la informacin para el servidor
				HashMap<String,String> lista = new HashMap<String,String>();
				lista.put("estadoActual", "estadoBuscandoPartida");
				GestorXML gestor = GestorXML.getGestorXML();
				String mensajeRespuesta = gestor.obtenerXML(lista);
				
				out.println(mensajeRespuesta);
			}
			
		}else if(estado == Cliente.ESTADO_ENPARTIDA){
			//Si esta en partida le pasamos el mensaje al gestor de partidas
			gestorPartidas.mensajeCliente(c, mensaje, out);			
		}			
	}	
	
}
