package Partida.Estados;

import java.io.PrintWriter;
import java.util.HashMap;

import GestionClientes.Cliente;

import Partida.Partida;
import Partida.GameObjets.Marcador;
import Utilidades.GestorXML;

public class EstadoFinPartida implements IEstado {
	
	private Partida partida;
	
	private Marcador marcador;
	
	public static final int VICTORIA_POR_PUNTOS = 0;
	public static final int VICTORIA_POR_DESCONEXION =1;
	private int causaFinPartida;
	private int kValor;
	
	//Constructor
	public EstadoFinPartida(Partida p) {
		this.partida = p;		
	}		
	//Setters
	public void iniciarFinPartida(Marcador m,int causa,Cliente c1,Cliente c2){
		this.marcador = m;
		this.causaFinPartida = causa;
		String valor = GestorXML.getGestorXML().obtenerValorConfiguracion("partida", "kValor");
		this.kValor = Integer.parseInt(valor);
		//Actualizamos el resultado de la partida en los perfiles de los jugadores
		actualizarResultadoClientes(c1, c2);
	}
	//Gestin de la solicitud de datos del cliente
	public void envioCliente(Cliente c, String msg, PrintWriter out) {
		//Obtenemos si el cliente solicita datos o enva datos
		String tipoMensaje = GestorXML.getGestorXML().obtenerValorXML(msg, "tipo");
		if(tipoMensaje.contentEquals("obtenerDatos")){
			//Creamos el fichero XML con la informacin para el cliente
			HashMap<String,String> lista = new HashMap<String,String>();			
			
			if(causaFinPartida == VICTORIA_POR_DESCONEXION){
				lista.put("causa", "desconexion");
				//Si es por desconexion se da por hecho el cliente que solicita la informacion
				//es el que queda en la partida
				lista.put("esGanador", String.valueOf(true));
			}else
			{
				boolean ganador = esGanadorPartida(c);
				lista.put("esGanador", String.valueOf(ganador));
				lista.put("causa", "puntos");
			}			
			
			lista.put("estadoActual", "estadoFinPartida");
			
			//Convertimos a XML y lo enviamos al cliente			
			String xml = GestorXML.getGestorXML().obtenerXML(lista);			
			out.println(xml);
		}
			
	}
	//Devuelve si el cliente es el ganador o no de la partida
	private boolean esGanadorPartida(Cliente c){
		int numCliente = partida.getIdCliente(c);
		int numGanador = marcador.getGanador();
		return numCliente == numGanador;
	}	
	
	private void actualizarResultadoClientes(Cliente c1,Cliente c2){		
		//Obtenemos el elo actual de cada jugador
		int eloJugador1 = c1.getPerfilJugador().getElo();
		int eloJugador2 = c2.getPerfilJugador().getElo();
		//Calculamos el resultado esperado de cada jugador entre 0 y 1.
		//El que mayor valor tenga se espera que gane.
		double ratingJugador1 = Math.pow(10, eloJugador1/400); 
		double ratingJugador2 = Math.pow(10, eloJugador2/400);
		double expectedJugador1 = ratingJugador1/(ratingJugador1 + ratingJugador2);
		double expectedJugador2 = ratingJugador2/(ratingJugador1 + ratingJugador2);
		//Finalmente actualizamos el elo
		int victoriaJugador1;
		int victoriaJugador2;
		if(esGanadorPartida(c1)){
			victoriaJugador1 = 1;
			victoriaJugador2 = 0;
		}else{
			victoriaJugador1 = 0;
			victoriaJugador2 = 1;
		}		
		int nuevoEloJugador1 = (int) (eloJugador1 + (kValor*(victoriaJugador1 - expectedJugador1)));
		int nuevoEloJugador2 = (int) (eloJugador2 + (kValor*(victoriaJugador2 - expectedJugador2)));
		
		c1.getPerfilJugador().actualizarPerfil(nuevoEloJugador1);
		c2.getPerfilJugador().actualizarPerfil(nuevoEloJugador2);
	}
	
	public void actualizar() {
		
	}

	

}
