package Partida.Estados;

import java.io.PrintWriter;
import java.util.HashMap;

import GestionClientes.Cliente;
import Partida.Partida;
import Utilidades.GestorXML;

public class EstadoIniciandoPartida implements IEstado{
	
	//Variables para controlar el tiempo hasta iniciar la partida
	private long tiempoInicio;
	private long tiempoMaximoInicio;
	
	private boolean inicioCuenta;
	
	private Partida partida;
	
	//Constructor
	public EstadoIniciandoPartida(Partida p) {
		this.partida = p;
		inicializar();
	}	
	
	private void inicializar() {		
		inicioCuenta = false;
		
		//Obtenemos el tiempo de la cuenta atras del fichero configuracin
		String tiempo = GestorXML.getGestorXML().obtenerValorConfiguracion("estadoIniciandoPartida", "tiempoMaximoInicio");
		this.tiempoMaximoInicio = Long.parseLong(tiempo);
	}

	public void iniciarCuentaAtras(){
		//Establecemos el tiempo inicio
		tiempoInicio = System.currentTimeMillis();		
		inicioCuenta = true;
	}
	//Gestin de la solicitud de datos del cliente
	public void envioCliente(Cliente c, String msg, PrintWriter out) {
		//Obtenemos si el cliente solicita datos o enva datos
		String tipoMensaje = GestorXML.getGestorXML().obtenerValorXML(msg, "tipo");
		//Si solicita datos
		if(tipoMensaje.contentEquals("obtenerDatos")){
			
			//Creamos el fichero XML con la informacin para el cliente
			HashMap<String,String> lista = new HashMap<String,String>();
			
			String tiempoRestante = String.valueOf(getTiempoRestante());
			lista.put("tiempoRestante", tiempoRestante);
			
			lista.put("estadoActual", "estadoIniciandoPartida");
			
			String xml = GestorXML.getGestorXML().obtenerXML(lista);
			
			//Lo enviamos al cliente
			out.println(xml);
		}
	//Este estado no requiere que de mensajes de otro tipo por parte del cliente		
	}
	
	//Devuelve en segundos el tiempo restante para finalizar la cuenta
	private int getTiempoRestante(){
		long tiempoTranscurrido = System.currentTimeMillis() - tiempoInicio;
		int tiempoRestante = (int)((tiempoMaximoInicio - tiempoTranscurrido)/1000);
		return tiempoRestante;
	}
	
	public void actualizar() {		
		//Si se ha iniciado la cuenta
		if(inicioCuenta){
			//Comprobamos si ha acabado la cuenta
			long tiempoTranscurrido = System.currentTimeMillis() - tiempoInicio;
			if(tiempoTranscurrido > tiempoMaximoInicio){
				//Cambiamos de estado a jugando				
				partida.setEstadoJugando();
			}
		}
		
	}

}
