package Partida.Estados;

import java.io.PrintWriter;
import java.util.HashMap;
import java.util.Random;

import GestionClientes.Cliente;
import Partida.Partida;
import Partida.GameObjets.Bola;
import Partida.GameObjets.Jugador;
import Partida.GameObjets.Marcador;
import Partida.GameObjets.Tablero;
import Utilidades.GestorXML;

public class EstadoJugando implements IEstado {

	private Jugador jugador1;
	private Jugador jugador2;
	
	private Tablero tablero;
	
	private Marcador marcador;
	
	private Bola bola;
	
	private Partida partida;
	
	//Constructor
	public EstadoJugando(Partida p) {
		this.partida = p;
		inicializar();
	}
	
	private void inicializar() {
		//Creamos los diferentes elementos
		tablero = new Tablero();
		marcador = new Marcador();
		crearJugadores();
		crearBola();		
	}
	//Getter
	public Marcador getMarcador(){
		return marcador;
	}
	//Inicializa la bola y asigna el saque a un jugador de forma random
	private void crearBola() {
		Random r = new Random();
		int num = r.nextInt(2);
		bola = new Bola();
		if(num == 1){
			bola.lanzarBolaDesdeJugador(jugador1);
		}else
		{
			bola.lanzarBolaDesdeJugador(jugador2);
		}
		
	}

	//Inicializa los jugadores y asigna a cada jugador de forma random un lado del tablero
	private void crearJugadores() {
		Random r = new Random();
		int num = r.nextInt(2);
		if(num == 1){
			jugador1 = new Jugador(Jugador.LADO_IZQUIERDO);
			jugador2 = new Jugador(Jugador.LADO_DERECHO);
		}else
		{
			jugador1 = new Jugador(Jugador.LADO_DERECHO);
			jugador2 = new Jugador(Jugador.LADO_IZQUIERDO);
		}
	}

	public void actualizar() {
		//Comprobamos si se ha acabado la partida
		if(marcador.hayGanador()){			
			//Cambiamos el estado			
			partida.setEstadoFinPartida(marcador,EstadoFinPartida.VICTORIA_POR_PUNTOS);			
		}else
		{	
			//Movemos la bola
			bola.moverBola();
			//Comprobamos si la bola va fuera del tablero y es punto para un jugador
			comprobarBolaFueraTablero();
			//Comprobamos si la bola colisiona con algun jugador
			comprobarColisionJugadores();			
		}
		
		
	}
	//Se comprueba si la bola colisiona con la raqueta de algun jugador
	private void comprobarColisionJugadores() {
		jugador1.comprobarColision(bola);
		jugador2.comprobarColision(bola);
	}
	//Se comprueba si algun jugador ha conseguido punto
	private void comprobarBolaFueraTablero(){
		int codigo = tablero.bolaOutOfBounds(bola);		
		if(codigo == Tablero.FUERA_ARRIBA){
			//Rebota la pelota si choca contra la pared de arriba
			bola.rebotar();			
		}else if(codigo == Tablero.FUERA_ABAJO){
			//Rebota la pelota si choca contra la pared de abajo			
			bola.rebotar();			
		}else if(codigo == Tablero.FUERA_IZQUIERDA){
			//El jugador de la derecha ha conseguido el punto
			//Comprobamos que jugador esta a la derecha			
			if(jugador1.getPosicionTablero() == Jugador.LADO_DERECHO){
				//Aumentamos sus puntos
				marcador.aumentarPuntos(1);
				//Sacamos la bola del lado del jugador contrario
				bola.lanzarBolaDesdeJugador(jugador2);
			}else
			{
				marcador.aumentarPuntos(2);
				bola.lanzarBolaDesdeJugador(jugador1);				
			}			
			
		}else if(codigo == Tablero.FUERA_DERECHA){		
			//Comprobamos que jugador esta a la izquierda
			if(jugador1.getPosicionTablero() == Jugador.LADO_IZQUIERDO){
				//Aumentamos sus puntos
				marcador.aumentarPuntos(1);
				//Sacamos la bola del lado del jugador contrario
				bola.lanzarBolaDesdeJugador(jugador2);
			}else
			{
				marcador.aumentarPuntos(2);
				bola.lanzarBolaDesdeJugador(jugador1);				
			}
		}		
	}
	
	//Gestin de la solicitud de datos del cliente
	public void envioCliente(Cliente c, String msg, PrintWriter out) {
		//Obtenemos si el cliente solicita datos o enva datos
		String tipoMensaje = GestorXML.getGestorXML().obtenerValorXML(msg, "tipo");
		if(tipoMensaje.contentEquals("obtenerDatos")){
			//Si el jugador solicita datos
			//Creamos el fichero XML con la informacin para el cliente
			HashMap<String,String> lista = new HashMap<String,String>();
			
			//Obtenemos el jugador del cual tenemos obtener los datos para el cliente que lo solicita
			int numCliente = partida.getIdCliente(c);
			if(numCliente == 1){
				//Si es el jugador 1 el que solicita le enviamos datos del jugador 2				
				lista.put("posYOponente", String.valueOf(jugador2.getPosY()));
				//Enviamos la puntuacin
				lista.put("puntuacionJugador", String.valueOf(marcador.getPuntuacionJugador(1)));
				lista.put("puntuacionOponente", String.valueOf(marcador.getPuntuacionJugador(2)));
				//Enviamos la informacin de la bola teniendo en cuando en cuenta que el jugador
				//en su pantalla siempre esta a la izquierda
				if(jugador1.getPosicionTablero() == Jugador.LADO_DERECHO){
					//Si esta en el lado derecho le mandamos la posicion en el eje x invertida					
					lista.put("bolaX", String.valueOf(bola.getPosXInvertida(tablero)));
					lista.put("bolaY", String.valueOf(bola.getPosY()));
				}else
				{
					lista.put("bolaX", String.valueOf(bola.getPosX()));
					lista.put("bolaY", String.valueOf(bola.getPosY()));
				}				
			}else
			{
				lista.put("posYOponente", String.valueOf(jugador1.getPosY()));
				lista.put("puntuacionJugador", String.valueOf(marcador.getPuntuacionJugador(2)));
				lista.put("puntuacionOponente", String.valueOf(marcador.getPuntuacionJugador(1)));
				if(jugador2.getPosicionTablero() == Jugador.LADO_DERECHO){
					lista.put("bolaX", String.valueOf(bola.getPosXInvertida(tablero)));
					lista.put("bolaY", String.valueOf(bola.getPosY()));					
				}else
				{
					lista.put("bolaX", String.valueOf(bola.getPosX()));
					lista.put("bolaY", String.valueOf(bola.getPosY()));
				}
			}
			//Enviamos el estado
			lista.put("estadoActual", "estadoJugando");
			//Enviamos direccion bola
			lista.put("direccionBolaX",String.valueOf(bola.getDireccionX()));
			//Convertimos a XML y lo enviamos al cliente			
			String xml = GestorXML.getGestorXML().obtenerXML(lista);			
			out.println(xml);
		}else if(tipoMensaje.contentEquals("envioDatos")){
			//Si el jugador enva datos en el mensaje
			//Obtenemos la posicin que enva el jugador
			String posicionY = GestorXML.getGestorXML().obtenerValorXML(msg, "posY");
			//Comprobamos que jugador es el que enva
			int numCliente = partida.getIdCliente(c);
			if(numCliente == 1){
				jugador1.setPosY(Double.parseDouble(posicionY));
			}else
			{
				jugador2.setPosY(Double.parseDouble(posicionY));
			}
			
		}else if(tipoMensaje.contentEquals("envioFuncion")){
			//Si el jugador enva una accin 
			//Obtenemos el identificador de la accin a realizar
			String idFuncion = GestorXML.getGestorXML().obtenerValorXML(msg, "idFuncion");
			if(idFuncion.contentEquals("pausar")){
				//Si el cliente solicita pausar la partida
				//Intentamos pausar la partida
				partida.setEstadoPausa(c);
			}
		}
	}

}
