package Partida.Estados;

import java.io.PrintWriter;
import java.util.HashMap;

import GestionClientes.Cliente;
import Partida.Partida;
import Utilidades.GestorXML;

public class EstadoPausa implements IEstado {

	
	//Variables para controlar el tiempo de pausa
	private long tiempoInicioPausa;
	private long tiempoMaximoPausa;
	
	//Variables para controlar quien ha pausado
	private boolean haPausadoJugador1;
	private boolean haPausadoJugador2;
	
	//Jugador que ha solicitado la pausa
	private int jugadorSolicitante;
	
	//Variables para controlar la reanudacin
	private boolean reaunandoPartida;
	private long tiempoInicioReanudarPartida;
	private long tiempoMaximoReanudacion;
	
	
	private Partida partida;
	
	public EstadoPausa(Partida p) {
		this.partida = p;
		inicializar();
	}
	
	private void inicializar() {
		haPausadoJugador1 = false;
		haPausadoJugador2 = false;
		reaunandoPartida = false;		
		//Obtenemos los valores del fichero configuracin
		String tiempo = GestorXML.getGestorXML().obtenerValorConfiguracion("estadoPausa", "tiempoMaximoPausa");
		this.tiempoMaximoPausa = Long.parseLong(tiempo);
		
		tiempo = GestorXML.getGestorXML().obtenerValorConfiguracion("estadoPausa", "tiempoMaximoReanudacion");
		this.tiempoMaximoReanudacion = Long.parseLong(tiempo);
	}	
		
	//Gestin de la solicitud de datos del cliente
	public void envioCliente(Cliente c, String msg, PrintWriter out) {
		//Obtenemos si el cliente solicita datos o enva datos
		String tipoMensaje = GestorXML.getGestorXML().obtenerValorXML(msg, "tipo");
		if(tipoMensaje.contentEquals("obtenerDatos")){
			//Si el jugador solicita datos
			//Creamos el fichero XML con la informacin para el cliente
			HashMap<String,String> lista = new HashMap<String,String>();
			
			boolean esSolicitante;
			int numJugador = partida.getIdCliente(c);
			if(numJugador == jugadorSolicitante){
				esSolicitante = true;
			}else{
				esSolicitante = false;
			}
			lista.put("esSolicitante",String.valueOf(esSolicitante));
			
			lista.put("reanudandoPartida", String.valueOf(reaunandoPartida));
			if(reaunandoPartida){
				lista.put("tiempoRestante", String.valueOf(getTiempoRestanteReanudacion()));
			}else
			{
				lista.put("tiempoRestante", String.valueOf(getTiempoRestantePausa()));
			}
			lista.put("estadoActual", "estadoPausa");
			
			//Convertimos a XML y lo enviamos al cliente			
			String xml = GestorXML.getGestorXML().obtenerXML(lista);			
			out.println(xml);
		}else if(tipoMensaje.contentEquals("envioFuncion")){
			//Si el jugador enva una accin 
			//Obtenemos el identificador de la accin a realizar
			String idFuncion = GestorXML.getGestorXML().obtenerValorXML(msg, "idFuncion");
			if(idFuncion.contentEquals("reanudar")){
				//Si el cliente solicita reanudar la partida
				//Obtenemos el nmero del jugador que solicita
				int numJugador = partida.getIdCliente(c);
				//Intentamos reanudar la partida
				intentarReanudarPartida(numJugador);				
			}
		}
		
		
	}

	//Devuelve true si se puede pausar la partida , false en caso contrario
	public boolean intentarPausar(int numJugador){
		//Comprobamos si el jugador ya haba pausado antes
		if(numJugador == 1){
			if(haPausadoJugador1){
				//Si ya ha pausado devolvemos false
				return false;
			}
				//Si no ha pausado iniciamos la pausa
				this.reaunandoPartida = false;
				this.haPausadoJugador1 = true;
				this.tiempoInicioPausa = System.currentTimeMillis();
				this.jugadorSolicitante = numJugador;
				
				return true;		
		}else
		{
			if(haPausadoJugador2){
				//Si ya ha pausado devolvemos false
				return false;
			}
				//Si no ha pausado iniciamos la pausa
				this.reaunandoPartida = false;
				this.haPausadoJugador2 = true;
				this.tiempoInicioPausa = System.currentTimeMillis();
				this.jugadorSolicitante = numJugador;
				
				return true;		
		}		
	}
	
	//Dado un jugador intenta reanudar la partida en caso de que el mismo haya solicidato la pausa previamente
	private void intentarReanudarPartida(int numJugador){
		//Comprobamos que sea el mismo jugador y que no se este ya reanudando
		if(numJugador == jugadorSolicitante && !reaunandoPartida){
			iniciarReanudacionPartida();			
		}
				
	}
	//Inicia la reanudacin de la partida
	private void iniciarReanudacionPartida(){		
		this.tiempoInicioReanudarPartida = System.currentTimeMillis();
		this.reaunandoPartida = true;
	}
	
	//Devuelve en segundos el tiempo restante de la pausa
	private int getTiempoRestantePausa(){			
		long tiempoTranscurrido = System.currentTimeMillis() - tiempoInicioPausa;
		int tiempoRestante = (int) ((tiempoMaximoPausa - tiempoTranscurrido)/1000);
		
		return tiempoRestante;		
	}
	
	//Devuelve en segundos el tiempo restante de la reanudacin
	private int getTiempoRestanteReanudacion(){				
		long tiempoTranscurrido = System.currentTimeMillis() - tiempoInicioReanudarPartida;
		int tiempoRestante = (int) ((tiempoMaximoReanudacion - tiempoTranscurrido)/1000);
			
		return tiempoRestante;		
	}	
	
	//Comprueba si ha finalizado la pausa
	private boolean comprobarFinPausa(){		
		//Obtenemos el tiempo transcurrido
		long tiempoTranscurrido = System.currentTimeMillis() - tiempoInicioPausa;		
		//Comprobamos si ha pasado el mximo de tiempo
		if(tiempoTranscurrido > tiempoMaximoPausa){
			return true;
		}else
		{
			return false;
		}		
	}
	//Comprueba si ha finalizado la reanudacion
	private boolean comprobarFinReanudacion(){
		//Obtenemos el tiempo transcurrido		
		long tiempoTranscurrido = System.currentTimeMillis() - tiempoInicioReanudarPartida;	
		//Comprobamos si ha pasado el mximo de tiempo
		if(tiempoTranscurrido > tiempoMaximoReanudacion){
			return true;
		}else
		{
			return false;
		}		
	}
	
	//Actulizacin del estado
	public void actualizar() {
		if(reaunandoPartida){
			//Si esta reanudando comprobamos si ya ha finalizado 
			boolean finReanudacion = comprobarFinReanudacion();
			if(finReanudacion){
				//Si ha finalizado cambiamos el estado de la partida a jugando
				partida.setEstadoJugando();
			}
			
		}else
		{
			//Si esta en pausa comprobamos si se ha terminado la pausa
			boolean finPausa = comprobarFinPausa();
			if(finPausa){
				//Si ha terminado iniciamos la reanudacin
				iniciarReanudacionPartida();
			}
		}
		
	}
	
	

}
