package Partida.GameObjets;

import java.awt.Dimension;
import java.util.Random;



import GestorPrincipal.GestorPrincipal;
import Utilidades.GestorXML;

public class Bola {
	
	public static final int COLISION_SUPERIOR = 0;
	public static final int COLISION_CENTRO = 1;
	public static final int COLISION_INFERIOR = 2;

	//Posicin bola
	private double posX;
	private double posY;
	
	//Control de velocidad
	private double velocidadActual;
	private double velocidadInicial;
	private double velocidadMaxima;
	private double aumentoVelocidad;	
	private int actualizacionesSegundo;
	
	//Vector direccin
	private double dX;
	private double dY;
	
	//Dimensiones 
	private double radio;
	
	//Constructor
	public Bola() {
		inicializar();
	}

	private void inicializar() {
		//Obtenemos las variables del fichero configuracin
		String radio = GestorXML.getGestorXML().obtenerValorConfiguracion("bola", "radio");
		this.radio = Double.parseDouble(radio);
		
		String velocidad =  GestorXML.getGestorXML().obtenerValorConfiguracion("bola", "velocidadInicial");
		this.velocidadInicial = Double.parseDouble(velocidad);
		this.velocidadActual = velocidadInicial;
		
		velocidad =  GestorXML.getGestorXML().obtenerValorConfiguracion("bola", "velocidadMaxima");
		this.velocidadMaxima = Double.parseDouble(velocidad);
		
		velocidad =  GestorXML.getGestorXML().obtenerValorConfiguracion("bola", "aumentoVelocidad");
		this.aumentoVelocidad = Double.parseDouble(velocidad);
		
		String as =  GestorXML.getGestorXML().obtenerValorConfiguracion("bola", "actualizacionesSegundo");
		this.actualizacionesSegundo = Integer.parseInt(as);	
		
	}
	
	//Setters
	public void setPosicion(double pX,double pY){
		this.posX = pX;
		this.posY = pY;
	}	
	
	public void setDireccion(double dX,double dY){
		//Normalizamos el vector para controlar la velocidad con la variable velocidad
		double longitud = Math.sqrt(Math.pow(dX, 2) + Math.pow(dY, 2));		
		this.dX = dX/longitud;
		this.dY = dY/longitud;		
	}
	
	//Getters
	public double getPosX() {
		return posX;
	}

	public double getPosY() {
		return posY;
	}

	public double getRadio() {
		return radio;
	}
	
	public double getDireccionX(){
		return dX;
	}
	
	//Devuelve la posicin en el eje x invertida al otro lado del tablero
	public double getPosXInvertida(Tablero t){
		Dimension d = t.getDimension();
		double ancho = d.getWidth();
		double mitad = ancho/2;
		double offset = posX - mitad;
		return mitad - offset;		
	}
	//Aumenta la velocidad de la bola
	private void aumentarVelocidad(){
		//Aumentamos la velocidad
		double aumento = aumentoVelocidad/actualizacionesSegundo;
		this.velocidadActual +=aumento;
		//Comprobamos que no se pase de la velocidad mxima
		if(velocidadActual > velocidadMaxima){
			velocidadActual = velocidadMaxima;
		}		
	}
	
	//Movimiento de bola
	public void moverBola(){
		//Movemos la bola en funcin de la velocidad y la direccin
		double v = velocidadActual/actualizacionesSegundo;
		double x = posX + (dX*v);
		double y = posY +(dY*v);
		
		//Establecemos la nueva posicin
		setPosicion(x, y);	
		
		//Aumentamos la velocidad
		aumentarVelocidad();
	}
	//Cambia la direccin de la y para simular el efecto rebote
	public void rebotar(){
		double nuevaDireccionY = -this.dY;
		setDireccion(dX, nuevaDireccionY);
	}
	
	//Cambia al direccin de la bola en funcion de donde ha colisionado con la paleta del jugador
	public void colisionJugador(int tipoColision) {
		switch (tipoColision) {
		case COLISION_SUPERIOR:
			setDireccion(-this.dX, -0.5);
			break;
		case COLISION_CENTRO:
			setDireccion(-this.dX, 0);
			break;
		case COLISION_INFERIOR:
			setDireccion(-this.dX, 0.5);
			break;		
		}	
		
	}
	
	//Lanza la bola desde un jugador en la direccin contraria para sacar	
	public void lanzarBolaDesdeJugador(Jugador j){
		//Obtenemos la direccin de lanzamiento
		double dY = 0;
		double dX = 0;
		int p = j.getPosicionTablero();
		if(p == Jugador.LADO_DERECHO){
			dX = -1.0;
		}else
		{
			dX = 1.0;
		}
		//La direccin Y lo obtenemos de forma random entre 0.5 y -0.5
		Random r = new Random();
		dY = r.nextDouble() - 1.0;
		//Reiniciamos la velocidad a la inicial
		this.velocidadActual = velocidadInicial;
		//Establecemos la direccin
		setDireccion(dX, dY);
		//Establecemos la posicin delante de la paleta del jugador
		double posX = j.getPosX();
		double posY = j.getPosY();
		double anchoPaleta = j.getAncho();
		if(p == Jugador.LADO_DERECHO){
			posX = posX - anchoPaleta - 3;
		}else
		{
			posX = posX + anchoPaleta + 3;
		}		
		setPosicion(posX, posY);
		
	}

	
	
	
}
