package Partida.GameObjets;



import Utilidades.GestorXML;

public class Jugador {
	
	private boolean colisionReciente;
	
	//Posicin jugador del centro de la paleta
	private double posX;
	private double posY;
	
	//Dimensiones paleta
	private double alto;
	private double ancho;
	
	//Posicin en tablero (lado derecho o izquierdo)
	private int posicionTablero;
	
	public static final int LADO_DERECHO = 0;
	public static final int LADO_IZQUIERDO = 1;
	
	//Constructor
	public Jugador(int posTablero) {		
		this.posicionTablero = posTablero;
		colisionReciente = false;
		inicializar();
	}

	private void inicializar() {
		//Cargamos las variables del fichero configuracin		
		String valor =  GestorXML.getGestorXML().obtenerValorConfiguracion("jugador", "alto");
		this.alto= Double.parseDouble(valor);
		
		valor =  GestorXML.getGestorXML().obtenerValorConfiguracion("jugador", "ancho");
		this.ancho= Double.parseDouble(valor);		
		
		//La posicin en el eje X siempre va ser fija dependiendo del lado que le toque
		//El jugador solo se mueve en el eje Y
		if(posicionTablero == LADO_DERECHO){			
			valor =  GestorXML.getGestorXML().obtenerValorConfiguracion("jugador", "posXDerecha");
			this.posX = Double.parseDouble(valor);
			
		}else if(posicionTablero == LADO_IZQUIERDO){			
			valor =  GestorXML.getGestorXML().obtenerValorConfiguracion("jugador", "posXIzquierda");
			this.posX = Double.parseDouble(valor);
		}
		valor =  GestorXML.getGestorXML().obtenerValorConfiguracion("jugador", "posYInicial");
		this.posY = Double.parseDouble(valor);		
	}
	//Getter
	public int getPosicionTablero(){
		return posicionTablero;
	}	
	
	public double getPosX() {
		return posX;
	}

	public double getPosY() {
		return posY;
	}

	public double getAlto() {
		return alto;
	}

	public double getAncho() {
		return ancho;
	}

	//Setter	
	public void setPosY(double pY){
		this.posY = pY;
	}	
	
	//Comprobamos si la bola colisiona con la paleta del jugador
	public void comprobarColision(Bola b){		
		//Obtenemos los datos de la bola
		double radio = b.getRadio();
		double bX = b.getPosX();
		double bY = b.getPosY();
		//Comprobamos si colisiona
		double rX = (posX-(ancho/2));
		double rY = (posY-(alto/2));
		double deltaX = bX - Math.max(rX, Math.min(bX, rX + ancho));
		double deltaY = bY - Math.max(rY, Math.min(bY, rY + alto));
		
		boolean colisiona = ((deltaX*deltaX) + (deltaY*deltaY)) < (radio*radio);
		
		if(colisiona){
			if(!colisionReciente){
				colisionReciente = true;
				//Comprobamos el punto colisin
				double diferenciaY = Math.abs(rY-bY);
				double longitudZona = alto/3;
				if(diferenciaY < longitudZona){
					b.colisionJugador(Bola.COLISION_SUPERIOR);
				}else if(diferenciaY > (longitudZona*2)){
					b.colisionJugador(Bola.COLISION_INFERIOR);
				}else{
					b.colisionJugador(Bola.COLISION_CENTRO);
				}				
			}			
		}else{
			colisionReciente = false;
		}
		
	}
}
