package Partida;

import java.io.PrintWriter;
import java.util.HashMap;

import GestionClientes.Cliente;

public class GestorPartidas {

	private HashMap<String,Partida> listaPartidas;	
	
	//Constructor
	public GestorPartidas() {
		inicializar();
	}
	
	private void inicializar() {
		listaPartidas = new HashMap<String,Partida>();		
	}
	
	//Gestin de mensajes de los clientes
	public void mensajeCliente(Cliente c,String msg,PrintWriter out){
		//Obtenemos la partida del cliente
		String id = c.getIdentificadorPartida();		
		synchronized (listaPartidas) {
			Partida p = listaPartidas.get(id);			
			//Le pasamos el mensaje a la partida para que lo gestione
			p.mensaje(c, msg, out);
		}	
		
	}
	
	//Crea una nueva partida entre dos clientes
	public void crearPartida(Cliente c1,Cliente c2){
		//Obtenemos un identificador nico para la partida juntando el identificador de ambos
		String id = c1.getKey() + c2.getKey();
		//Lo guardamos en el cliente
		c1.setIdentificadorPartida(id);
		c2.setIdentificadorPartida(id);
		//Creamos la partida y la metemos en la lista
		synchronized (listaPartidas) {
			Partida p = new Partida(c1, c2);			
			listaPartidas.put(id, p);
			//Inicamos el thread de la partida
			p.iniciarThread();
			//Cambiamos el estado de los clientes
			c1.setEstado(Cliente.ESTADO_ENPARTIDA);
			c2.setEstado(Cliente.ESTADO_ENPARTIDA);
		}		
	}
	//Elimina una partida dado un identificador
	public void eliminarPartida(String key){
		synchronized (listaPartidas) {
			listaPartidas.remove(key);			
		}		
	}	
}
