package Partida;

import java.io.PrintWriter;

import GestionClientes.Cliente;
import GestionClientes.DatosCliente;
import GestorPrincipal.GestorPrincipal;
import Partida.Estados.EstadoFinPartida;
import Partida.Estados.EstadoIniciandoPartida;
import Partida.Estados.EstadoJugando;
import Partida.Estados.EstadoPausa;
import Partida.Estados.IEstado;
import Partida.GameObjets.Marcador;
import Utilidades.GestorXML;

public class Partida implements Runnable {

	
	private Cliente cliente1;
	private Cliente cliente2;
	
	private Thread thread;
	
	private int actualizacionesPorSegundo;
	
	//Estados de la partida
	private static final int ESTADO_INICIANDOPARTIDA = 0;
	private static final int ESTADO_JUGANDO = 1;
	private static final int ESTADO_PAUSA = 2;
	private static final int ESTADO_FINPARTIDA = 3;
	private static final int NUM_ESTADOS = 4;
	private IEstado[] listaEstados;
	private int estadoActual;
	
	private boolean fin;
	
	//Constructor
	public Partida(Cliente c1,Cliente c2) {
		this.cliente1 = c1;
		this.cliente2 = c2;
		inicializar();
	}	
	
	private void inicializar() {
		//Obtenemos la configuracin del fichero XML
		String s = GestorXML.getGestorXML().obtenerValorConfiguracion("partida", "actualizacionesSegundo");
		this.actualizacionesPorSegundo = Integer.parseInt(s);	
		
		//Iniciamos la lista de estados
		listaEstados = new IEstado[NUM_ESTADOS];
		listaEstados[ESTADO_INICIANDOPARTIDA] = new EstadoIniciandoPartida(this);
		listaEstados[ESTADO_JUGANDO] = new EstadoJugando(this);
		listaEstados[ESTADO_PAUSA] = new EstadoPausa(this);
		listaEstados[ESTADO_FINPARTIDA] = new EstadoFinPartida(this);
		
		//Establecemos el estado inicio partida
		setEstadoIniciandoPartida();		
	}	
	//Start thread
	public void iniciarThread(){		
		fin = false;
		if(thread ==null){
			thread = new Thread(this);
		}		
		thread.start();
	}
	//Stop thread
	private void pararThread(){
		fin = true;
		if(thread != null){
			thread.interrupt();
		}
		//Eliminamos la partida del gestor
		GestorPrincipal gestorPrincipal = GestorPrincipal.getGestorPrincipal();
		String idPartida = cliente1.getIdentificadorPartida();
		gestorPrincipal.eliminarPartida(idPartida);
	}
	//Devuelve si un cliente tiene el id 1 o 2 de la partida
	public int getIdCliente(Cliente c){
		if(cliente1.equals(c)){
			return 1;
		}else
		{
			return 2;
		}
	}
	//Getters
	public IEstado getEstado(int estado){
		return listaEstados[estado];
	}
	public boolean haFinalizado(){
		return fin;
	}
	
	//Setters estados
	public void setEstadoIniciandoPartida(){
		//Establecemos el estado iniciando partida
		((EstadoIniciandoPartida) getEstado(ESTADO_INICIANDOPARTIDA)).iniciarCuentaAtras();
		estadoActual = Partida.ESTADO_INICIANDOPARTIDA;
	}
	public void setEstadoJugando(){		
		//Establecemos el estado jugando
		estadoActual = Partida.ESTADO_JUGANDO;
	}
	public synchronized void setEstadoPausa(Cliente c){
		//Comprobamos que no estamos en pausa
		if(estadoActual != ESTADO_PAUSA){
			//Obtenemos el nmero del jugador que solicita la pausa
			int numJugador = getIdCliente(c);
			//Solicitamos al estadoPausa si puede pausar la partida
			boolean puedePausar = ((EstadoPausa) getEstado(ESTADO_PAUSA)).intentarPausar(numJugador);
			if(puedePausar){
				//Si se puede cambiamos el estado
				estadoActual = Partida.ESTADO_PAUSA;
			}
		}		
	}
	public void setEstadoFinPartida(Marcador m,int causa){
		//Iniciamos el estado fin partida
		((EstadoFinPartida) getEstado(ESTADO_FINPARTIDA)).iniciarFinPartida(m, 
				causa,cliente1, cliente2);
		//Cambiamos al estado fin partida
		estadoActual = Partida.ESTADO_FINPARTIDA;		
	}
	
	//Recibe un mensaje del cliente
	public void mensaje(Cliente c,String msg, PrintWriter out){
		//Le pasamos el mensaje al estado actual	
		IEstado estado = getEstado(estadoActual);		
		estado.envioCliente(c, msg, out);
	}
	//Comprueba si ambos clientes siguen conectados
	private void comprobarConexionClientes(){
		//Comprobamos que no estemos ya en el estado fin partida
		if(estadoActual != ESTADO_FINPARTIDA){
			//Pasamos al estado fin partida si algn cliente se ha desconectado
			if(cliente1.getEstado() == Cliente.ESTADO_DESCONECTADO){			
				Marcador m = ((EstadoJugando) getEstado(ESTADO_JUGANDO)).getMarcador();
				setEstadoFinPartida(m, EstadoFinPartida.VICTORIA_POR_DESCONEXION);
			}else if(cliente2.getEstado() == Cliente.ESTADO_DESCONECTADO){
				Marcador m = ((EstadoJugando) getEstado(ESTADO_JUGANDO)).getMarcador();
				setEstadoFinPartida(m, EstadoFinPartida.VICTORIA_POR_DESCONEXION);
			}
		}		
	}
	//Comprueba si los dos clientes ya se han desconectado
	private void comprobarFin(){
		//Si ambos clientes estan desconectados paramos el thread
		if((cliente1.getEstado() == Cliente.ESTADO_DESCONECTADO)&&
				(cliente2.getEstado() == Cliente.ESTADO_DESCONECTADO)){			
			pararThread();
		}
	}
	
	//Loop de la partida
	public void run() {	
		
		long lastTime = System.nanoTime();
		double amountOfTicks = actualizacionesPorSegundo;
		double ns = 1000000000 / amountOfTicks;
		double delta = 0;	
				
		while(!fin){
			//Control para mantener las actualizaciones por segundo establecidas
			long now = System.nanoTime();
			delta += (now - lastTime) / ns;
			lastTime = now;
			while(delta >= 1){				
				//Comprobar fin
				comprobarFin();
				//Comprobamos si algun cliente se ha desconectado y se le otorga la victoria al otro
				comprobarConexionClientes();
				//Actualizamos el estado actual
				getEstado(estadoActual).actualizar();				
				delta--;				
			}			
		}
		
	}

}
