package SistemaEmparejamiento;

import java.util.ArrayList;

import Utilidades.GestorXML;
import GestionClientes.Cliente;
import GestorPrincipal.GestorPrincipal;

public class GestorEmparejamientos implements Runnable{

	
	private ArrayList<Cliente> listaClientes;
	
	private Thread thread;	
	
	private boolean fin;
	
	private long sleepTime;
	
	//Indica el tiempo mximo que un cliente puede estar buscando si no encuentra gente en su rango
	//Si se supera el tiempo buscar gente de cualquier rango
	private long tiempoMaximoEnCola;
	
	//Indica la diferencia de elo que puede haber entre jugadores para poder emparejarlos
	private int diferenciaElo;
	
	//Constructor
	public GestorEmparejamientos() {
		inicializar();
	}

	private void inicializar() {
		listaClientes = new ArrayList<Cliente>();
		
		//Cargamos los valores del fichero configuracin
		String s = GestorXML.getGestorXML().obtenerValorConfiguracion("sistemaEmparejamiento", "sleepTime");
		this.sleepTime = Long.parseLong(s);
		
		s = GestorXML.getGestorXML().obtenerValorConfiguracion("sistemaEmparejamiento", "diferenciaElo");
		this.diferenciaElo = Integer.parseInt(s);
		
		s = GestorXML.getGestorXML().obtenerValorConfiguracion("sistemaEmparejamiento", "tiempoMaximoEnCola");
		this.tiempoMaximoEnCola = Long.parseLong(s);
		
		//Iniciamos el thread
		iniciarThread();
	}
	//Inicia el thread
	private void iniciarThread(){
		if(thread == null){
			thread = new Thread(this);
		}
		fin = false;
		thread.start();
	}
	//Para el thread
	private void pararThread(){
		fin = true;
		if(thread != null){
			thread.interrupt();
		}
	}
	//Aade un cliente a la lista
	public void addCliente(Cliente c){
		synchronized (listaClientes) {
			listaClientes.add(c);			
			c.setEstado(Cliente.ESTADO_BUSCANDO);
		}		
	}
	//Elimina un cliente de la lista
	public void eliminarCliente(Cliente c){
		synchronized (listaClientes) {
			listaClientes.remove(c);
		}		
	}
	
	//Intenta emparejar el cliente con otro de la lista 
	private boolean intentarEmparejar(Cliente c){
		//Comprobamos si el jugador lleva demasiado tiempo en cola
		long tiempoConexion = c.getTiempoInicioConexion();		
		long tiempoTranscurrido = System.currentTimeMillis() -tiempoConexion;		
		if(tiempoTranscurrido > tiempoMaximoEnCola){
			return intentarEmparejarSinElo(c);			
		}else
		{
			return intentarEmparejarConElo(c);
		}
	}
	//Intenta emparejar al cliente teniendo en cuenta la diferencia 
	//de elo minima para un emparejamiento
	private boolean intentarEmparejarConElo(Cliente c){
		double eloCliente = c.getPerfilJugador().getElo();
		
		Cliente mejorCandidato = null;
		double mejorDiferencia = 10000000;
		
		//Recorremos la lista de clientes
		for(Cliente c2 : listaClientes){
			//Comprobamos que no sea el mismo cliente
			if(c != c2){
				//Obtenemos la diferencia de elo entre los clientes
				double eloC2 = c2.getPerfilJugador().getElo();
				double diferencia = Math.abs(eloCliente -eloC2);
				//Comprobamos que se cumpla que la diferencia es menor que la permitida y que sea menor que otra 
				//diferencia vlida encontrada previamente
				if((diferencia <= diferenciaElo) && (diferencia < mejorDiferencia)){
					//Actualizamos punteros
					mejorCandidato = c2;
					mejorDiferencia = diferencia;
				}
			}
		}
		//Si se ha encontrado un candidato vlido
		if(mejorCandidato != null){
			GestorPrincipal gestorPrincipal = GestorPrincipal.getGestorPrincipal();
			gestorPrincipal.crearPartida(c, mejorCandidato);
			//Los eliminamos de la lista
			eliminarCliente(mejorCandidato);
			eliminarCliente(c);
			return true;
		}else{
			return false;
		}
		
	}
	//Intenta emparejar al cliente sin tener en cuenta la diferencia 
	//de elo minima para un emparejamiento
	private boolean intentarEmparejarSinElo(Cliente c){
		double eloCliente = c.getPerfilJugador().getElo();
		
		Cliente mejorCandidato = null;
		double mejorDiferencia = 10000000;
		
		//Recorremos la lista de clientes
		for(Cliente c2 : listaClientes){
			//Comprobamos que no sea el mismo cliente
			if(c != c2){
				//Obtenemos la diferencia de elo entre los clientes
				double eloC2 = c2.getPerfilJugador().getElo();
				double diferencia = Math.abs(eloCliente -eloC2);
				//Si es la menor encontrada
				if(diferencia < mejorDiferencia){
					//Actualizamos los punteros
					mejorCandidato = c2;
					mejorDiferencia = diferencia;
				}
			}
		}
		
		//Si se ha encontrado un candidato vlido
		if(mejorCandidato != null){
			GestorPrincipal gestorPrincipal = GestorPrincipal.getGestorPrincipal();
			gestorPrincipal.crearPartida(c, mejorCandidato);
			//Los eliminamos de la lista
			eliminarCliente(mejorCandidato);
			eliminarCliente(c);
			return true;
		}else{
			return false;
		}
		
	}
	
	//Loop del thread
	public void run() {
		
		while(!fin){			
			//Comprobamos que al menos haya dos clientes en la lista
			synchronized (listaClientes) {
				if(listaClientes.size() > 1){
					//Recorremos la lista de clientes
					for(Cliente c : listaClientes){
						//Intentamos emparejar el cliente
						boolean emparejado = intentarEmparejar(c);
						if(emparejado){
							break;
						}
					}
				}	
			}					
			//Pausamos el thread durante un corto periodo tiempo
			//para que el procesador atienda los demas threads
			try {
				thread.sleep(sleepTime);
			} catch (InterruptedException e) {				
				e.printStackTrace();
			}
		}
	}
	
	
}
