package Modelo;

import java.awt.image.BufferedImage;
import java.io.File;
import java.util.ArrayList;

import javax.imageio.ImageIO;

public class GestorJuegos {

	private static GestorJuegos miGestorJuegos;
	
	private ArrayList<Juego> listaJuegos;
	
	private GestorJuegos(){
		cargarListaJuegos();		
	}	
	

	public static GestorJuegos getGestorJuegos(){
		if(miGestorJuegos == null){
			miGestorJuegos = new GestorJuegos();
		}
		return miGestorJuegos;
	}
	
	//Carga la lista de juegos disponibles en el directorio
	private void cargarListaJuegos() {
		listaJuegos = new ArrayList<Juego>();
		try {
			//Obtenemos la direccion donde estan los juegos
			File f1 = new File(System.getProperty("java.class.path"));
			File f2 = f1.getAbsoluteFile().getParentFile();
			String direccionJuegos = f2.getAbsolutePath()+"/Games";			
			File ficheroJuegos = new File(direccionJuegos);
			//Recorremos cada carpeta que reprensenta un juego
			for (File carpeta : ficheroJuegos.listFiles()) {
				String nombreJuego = carpeta.getName();
				String pathJuego = "";
				BufferedImage imagen = null;
				//Recorremos cada fichero de la carpeta
				for (File f : carpeta.listFiles()) {		        
			        //Comprobamos si es la imagen o el ejecutable
			        String n = f.getName();
			        if(n.contentEquals("exe.jar")){
			        	pathJuego = f.getAbsolutePath();			        	
			        }else if(n.contentEquals("image.png")){
			        	imagen = ImageIO.read(f);
			        }
			    }
				//Creamos el juego y lo aadimos a la lista
				Juego j = new Juego(nombreJuego, pathJuego, imagen);
				listaJuegos.add(j);
		    }
			
		} catch (Exception e) {
			e.printStackTrace();
		}
		
	}
	
	//Devuelve una lista con los nombres de los juegos
	public String[] getNombreJuegos(){
		String[] listaNombres = new String[listaJuegos.size()];
		for (int i = 0; i < listaJuegos.size(); i++) {
			String nombre = listaJuegos.get(i).getNombre();
			listaNombres[i] = nombre;
		}
		return listaNombres;
	}	
	//Ejecuta un juego dado el nombre
	public void ejecutarJuego(String nombreJuego){
		//Buscamos el juego con el nombre y lo ejecutamos
		for (Juego juego : listaJuegos) {
			boolean b = juego.coincideNombre(nombreJuego);
			if(b){
				juego.ejecutarJuego();
				break;
			}
		}
	}
	//Devuelve la imagen de un juego
	public BufferedImage getImagenJuego(String nombreJuego){
		BufferedImage imagen = null;
		//Buscamos el juego con el nombre y lo ejecutamos
		for (Juego juego : listaJuegos) {
			boolean b = juego.coincideNombre(nombreJuego);
			if(b){
				imagen = juego.getImagen();
				break;
			}
		}
		return imagen;
	}
}
