package SpaceInvaders.Estados;

import java.awt.Dimension;
import java.awt.Graphics;
import java.util.ArrayList;
import java.util.Random;


import SpaceInvaders.GameplayObjects.Bala;
import SpaceInvaders.GameplayObjects.Explosion;

import SpaceInvaders.GameplayObjects.Bonificaciones.Bonificacion;
import SpaceInvaders.GameplayObjects.Bonificaciones.BonificacionDisparoRapido;
import SpaceInvaders.GameplayObjects.Bonificaciones.BonificacionEscudo;
import SpaceInvaders.GameplayObjects.Bonificaciones.BonificacionTripleDisparo;
import SpaceInvaders.GameplayObjects.Bonificaciones.BonificacionVida;
import SpaceInvaders.GameplayObjects.Enemigos.Enemigo;
import SpaceInvaders.GameplayObjects.Enemigos.SistemaSpawn;
import SpaceInvaders.GameplayObjects.Jugador.Jugador;
import SpaceInvaders.GestorJuego.GestorPrincipal;
import SpaceInvaders.InferfazUsuario.IUIListener;
import SpaceInvaders.InferfazUsuario.IconoSonido;
import SpaceInvaders.InferfazUsuario.Imagen;
import SpaceInvaders.InferfazUsuario.UIObject;
import SpaceInvaders.InferfazUsuario.UIObjectInteractive;
import SpaceInvaders.Sonido.GestorSonido;
import SpaceInvaders.Utilidades.GestorXML;

public class EstadoJugar implements IState, IUIListener{

	private Dimension dimensionPantalla;	
	
	private ArrayList<UIObject> listaObjetosInterfaz;
	private ArrayList<Bonificacion> listaBonificaciones;
	private ArrayList<Bala> listaBalas;
	private ArrayList<Explosion> listaExplosiones;
	
	private Jugador jugador;
	
	private ArrayList<Enemigo> listaEnemigos;
	private SistemaSpawn sistemaSpawn;	
	
	
	public EstadoJugar() {			
		inicializar();		
	}	
	
	private void inicializar(){
		listaObjetosInterfaz = new ArrayList<UIObject>();
		listaBonificaciones = new ArrayList<Bonificacion>();
		listaBalas = new ArrayList<Bala>();
		listaEnemigos = new ArrayList<Enemigo>();
		listaExplosiones = new ArrayList<Explosion>();
		
		this.dimensionPantalla = GestorPrincipal.getGestorPrincipal().getDimensionPantalla();
		
		Imagen fondo = new Imagen(0, 0, (int)dimensionPantalla.getWidth(),(int)dimensionPantalla.getHeight(), "/Imagenes/background.png");
		listaObjetosInterfaz.add(fondo);
		
		//Creamos el icono de sonido
		IconoSonido is = new IconoSonido(460, 660, "sonido", 30, 30);
		is.addListener(this);
		listaObjetosInterfaz.add(is);
		
		crearJugador();		
		sistemaSpawn = new SistemaSpawn(this);	
		
		
	}
	
	public Dimension getDimensionPantalla(){
		
		return dimensionPantalla;
	}
	
	public Jugador getJugador(){
		return jugador;
	}
	
	
	private void crearJugador(){
		//Obtenemos la posicin inicial
		String s = GestorXML.getGestorXML().obtenerValorConfiguracion("naveJugador","posXInicial");
		double x = Double.parseDouble(s);
		s = GestorXML.getGestorXML().obtenerValorConfiguracion("naveJugador","posYInicial");
		double y = Double.parseDouble(s);
		jugador = new Jugador(x, y, this);
	}
	
	private void actualizarBalas(){
		//Actualizamos cada bala
		for(Bala b : listaBalas){
			b.update();
		}
		//Eliminamos las balas finalizadas
		for(int i = 0 ; i < listaBalas.size() ; i++){
			Bala bala = listaBalas.get(i);
			boolean fin = bala.getFin();
			if(fin){
				listaBalas.remove(listaBalas.get(i));				
			}
		}	
	}
	
	private void actualizarEnemigos(){
		//Actualizamos enemigos
		for(Enemigo e : listaEnemigos){
			e.update();
		}
		//Eliminamos enemigos finalizados
		for(int i = 0 ; i < listaEnemigos.size() ; i++){
			if(listaEnemigos.get(i).getFin()){
				listaEnemigos.remove(listaEnemigos.get(i));
			}
		}
	}
	
	private void actualizarExplosiones() {
		//Actualizamos explosiones
		for(Explosion e : listaExplosiones){
			e.update();
		}
		//Eliminamos explosiones finalizados
		for(int i = 0 ; i < listaExplosiones.size() ; i++){
			if(listaExplosiones.get(i).getFin()){
				listaExplosiones.remove(listaExplosiones.get(i));
			}
		}		
	}
	
	private void actualizarSistemaSpawn(){
		sistemaSpawn.update();
		//Comprobamos si se ha acabado la oleada
		boolean finOleada = listaEnemigos.size() == 0 && sistemaSpawn.finOleada();
		if(finOleada){
			sistemaSpawn.nuevaOleda();			
		}
	}
	private void actualizarInterfaz() {
		for (UIObject o : listaObjetosInterfaz) {
			o.update();
		}		
	}
	
	private void actualizarBonificaciones() {
		for(Bonificacion b : listaBonificaciones){
			b.update();
		}
		for(int i = 0 ; i < listaBonificaciones.size() ; i++){
			if(listaBonificaciones.get(i).getFin()){
				listaBonificaciones.remove(listaBonificaciones.get(i));
			}
		}
		
	}
	
	public void update() {
		actualizarInterfaz();
		//Actualizamos sistema spawn
		actualizarSistemaSpawn();
		//Actualizamos balas
		actualizarBalas();
		//Actualizamos enemigos
		actualizarEnemigos();
		//Actualizar explosiones
		actualizarExplosiones();
		//Actualizamos bonificaciones
		actualizarBonificaciones();
			
		jugador.update();				
	}
	

	public void jugadorMuerto(){		
		//Cambiamos al estado fin partida
		GestorPrincipal.getGestorPrincipal().partidaFinalizada(jugador.getPuntuacion());		
	}
	
	
	public void addEnemigo(Enemigo e){
		listaEnemigos.add(e);
	}
	
	public void addBala(Bala b){
		listaBalas.add(b);
	}
	
	public void addExplosion(Explosion e){
		listaExplosiones.add(e);
	}
	
	public ArrayList<Enemigo> getListaEnemigos() {
		return listaEnemigos;
	}	
	
	//Aade una bonificacion random
	public void addBonificacionRandom(double x,double y){		
		Random r = new Random();
		Bonificacion b = null;		
		b= new BonificacionDisparoRapido(x,y,this);
		int n = r.nextInt(Bonificacion.NUM_BONIFICACIONES);
		switch (n) {
		case 0:
			b = new BonificacionVida(x,y,this);
			break;
		case 1:
			b = new BonificacionDisparoRapido(x,y,this);
			break;		
		case 2:
			b = new BonificacionEscudo(x,y,this);
			break;
		case 3:
			b = new BonificacionTripleDisparo(x, y, this);	
		}
		listaBonificaciones.add(b);
	}

	
	public void render(Graphics g) {
		//Dibujamos cada objeto de la interfaz
		for (UIObject u : listaObjetosInterfaz) {
			u.draw(g);
		}		
		//Dibujamos los enemigos
		for(Enemigo e : listaEnemigos){
			e.draw(g);
		}
		//Dibujamos cada bala
		for(Bala b : listaBalas){
			b.draw(g);
		}		
		//Dibujamos explosiones
		for (Explosion e : listaExplosiones) {
			e.draw(g);
		}
		//Dibujamos bonificaciones
		for(Bonificacion b : listaBonificaciones){
			b.draw(g);
		}
		jugador.draw(g);	
		
	}
	
	public void onSet() {
		//Ponemos musica
		GestorSonido.getGestorSonido().ponerMusica(GestorSonido.MUSICA_JUGAR);
	}

	public void onChange() {
		//Quitamos musica
		GestorSonido.getGestorSonido().detenerMusica();
	}

	
	public void interaccion(UIObjectInteractive object) {
		String id = object.getIdentificador();
		if(id.contentEquals("sonido")){
			GestorSonido.getGestorSonido().ponerMusica(GestorSonido.MUSICA_JUGAR);
		}		
	}

}
