package SpaceInvaders.Estados;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.util.ArrayList;

import SpaceInvaders.GestorJuego.GestorPrincipal;
import SpaceInvaders.InferfazUsuario.BotonInterfaz;
import SpaceInvaders.InferfazUsuario.IUIListener;
import SpaceInvaders.InferfazUsuario.IconoSonido;
import SpaceInvaders.InferfazUsuario.Imagen;
import SpaceInvaders.InferfazUsuario.Texto;
import SpaceInvaders.InferfazUsuario.UIObject;
import SpaceInvaders.InferfazUsuario.UIObjectInteractive;

import SpaceInvaders.Utilidades.GestorUsuarios;
import SpaceInvaders.Utilidades.Usuario;

public class EstadoVerRanking implements IState,IUIListener{

	//Objetos para interfaz general
	private ArrayList<UIObject> listaObjetosInterfaz;
	
	//Objetos para dibujar el ranking
	private ArrayList<UIObject> listaObjetosRanking;
	ArrayList<Usuario> listaRanking;	
	
	//Constructor
	public EstadoVerRanking() {		
		crearInterfaz();
	}
	
	private void crearInterfaz() {		
		listaObjetosInterfaz = new ArrayList<UIObject>();
		//Creamos el fondo
		Dimension d = GestorPrincipal.getGestorPrincipal().getDimensionPantalla();			
		Imagen fondo = new Imagen(0, 0, (int)d.getWidth(),(int)d.getHeight(), "/Imagenes/background.png");
		listaObjetosInterfaz.add(fondo);
		
		//Ttulo
		Texto t = new Texto(250, 120, "RANKING", Texto.CENTER_ALIGNMENT, false, Color.GREEN, 60);
		listaObjetosInterfaz.add(t);
		
		//Botones
		BotonInterfaz bi = new BotonInterfaz(250, 650, "atras", "BACK",30);
		bi.addListener(this);
		listaObjetosInterfaz.add(bi);
		
		//Creamos el icono de sonido
		IconoSonido is = new IconoSonido(460, 660, "sonido", 30, 30);
		is.addListener(this);
		listaObjetosInterfaz.add(is);
		
		//Creamos la lista ranking		
		crearListaRanking();
		
	}
	//Crea los objetos para mostrar el ranking
	private void crearListaRanking(){
		//Creamos una nueva lista
		listaObjetosRanking = new ArrayList<UIObject>();
		//Cargamos la lista ranking
		cargarRanking();
		
		//Variables para colocar los textos
		int posXNombre = 30;
		int posXPuntuacion = 400;
		int posY = 180;
		int aumentoY = 40;
		
		boolean usuarioActualEnRanking = false;
		
		int size = listaRanking.size();
		
		//Recorremos cada usuario
		for(int i = 0 ; i < 10 ;i++){
			//Comprobamos que haya usuarios en la lista
			if(i >= size){
				//Si no hay lo indicamos con XXXX
				String indice = String.valueOf(i+1+". ");
				Texto t = new Texto(posXNombre,posY, indice +"XXXX", Texto.LEFT_ALIGNMENT, false, Color.white, 30);
				listaObjetosRanking.add(t);				
				Texto t2 = new Texto(posXPuntuacion,posY, "XXXX", Texto.CENTER_ALIGNMENT, false, Color.white, 30);
				listaObjetosRanking.add(t2);
			}else
			{
				//Obtenemos el usuario con el nombre y puntuacin
				Usuario u = listaRanking.get(i);
				String indice = String.valueOf(i+1+". ");
				String nombre = u.getIdentificador();
				int puntos = u.getMejorPuntuacion();
				//Comprobamos si es el usuario actual
				if(GestorUsuarios.getGestorUsuarios().esUsuarioActual(u)){
					//Lo ponemos en rojo el usuario actual
					usuarioActualEnRanking = true;
					Texto t = new Texto(posXNombre,posY, indice + "YOU", Texto.LEFT_ALIGNMENT, false, Color.RED, 30);
					listaObjetosRanking.add(t);
					
					Texto t2 = new Texto(posXPuntuacion,posY, String.valueOf(puntos), Texto.CENTER_ALIGNMENT, false, Color.RED, 30);
					listaObjetosRanking.add(t2);
				}else
				{
					Texto t = new Texto(posXNombre,posY, indice + nombre, Texto.LEFT_ALIGNMENT, false, Color.white, 30);
					listaObjetosRanking.add(t);
					
					Texto t2 = new Texto(posXPuntuacion,posY, String.valueOf(puntos), Texto.CENTER_ALIGNMENT, false, Color.white, 30);
					listaObjetosRanking.add(t2);
				}				
				
			}
			
			
			posY += aumentoY;
		}
		//Si el usuario actual no esta en ranking lo ponemos debajo
		if(!usuarioActualEnRanking){
			posY += 40;
			Texto t = new Texto(posXNombre,posY, "TU", Texto.LEFT_ALIGNMENT, false, Color.red, 30);
			listaObjetosRanking.add(t);
			int puntos = GestorUsuarios.getGestorUsuarios().getUsuarioActual().getMejorPuntuacion();
			Texto t2 = new Texto(posXPuntuacion,posY, String.valueOf(puntos), Texto.CENTER_ALIGNMENT, false, Color.red, 30);
			listaObjetosRanking.add(t2);
		}		
	}
	
	//Obtiene la lista usuarios para el ranking
	private void cargarRanking(){
		listaRanking = GestorUsuarios.getGestorUsuarios().getRankingUsuarios();
	}
	
	//Refrescamos el ranking por si ha habido cambios
	public void onSet() {		
		crearListaRanking();
	}	
	public void onChange() {}
	
	//Actualizamos
	public void update() {
		for (UIObject uiObject : listaObjetosInterfaz) {
			uiObject.update();
		}		
	}
	//Dibujamos
	public void render(Graphics g) {		
		for (UIObject u1 : listaObjetosInterfaz) {
			u1.draw(g);
		}	
		for (UIObject u2 : listaObjetosRanking) {
			u2.draw(g);
		}	
	}

	
	public void interaccion(UIObjectInteractive object) {
		String id = object.getIdentificador();
		if(id.contentEquals("atras")){			
			GestorPrincipal.getGestorPrincipal().cambiarEstado(StateMachine.MENU_INICIO);
		}		
	}
	
	
}
