package SpaceInvaders.Estados;


import java.awt.Graphics;



public class StateMachine {

	//Listado estados posibles
	public static final int MENU_INICIO = 0;
	public static final int JUGAR = 1;
	public static final int VER_RANKING = 2;
	public static final int INSTRUCCIONES = 3;
	public static final int FIN_PARTIDA = 4;
	public static final int PAUSA = 5;
	
	private static final int NUM_ESTADOS = 6;
	
	private IState[] listaEstados;
	
	private int estadoActual;
	
	//Constructor
	public StateMachine() {
		inicilizar();		
	}	
	private void inicilizar() {
		crearEstados();
		//Ponemos el estado inicial
		setState(MENU_INICIO);		
	}	
	
	
	private void crearEstados(){
		listaEstados = new IState[NUM_ESTADOS];		
		listaEstados[MENU_INICIO] = new EstadoMenuInicio();			
		listaEstados[JUGAR] = new EstadoJugar();		
		listaEstados[VER_RANKING] = new EstadoVerRanking();		
		listaEstados[INSTRUCCIONES] = new EstadoInstrucciones();
		listaEstados[PAUSA] = new EstadoPausa();
	}	
	
	public void nuevoEstadoPartida(){
		if(listaEstados[estadoActual] != null){
			listaEstados[estadoActual].onChange();
		}		
		listaEstados[JUGAR] = new EstadoJugar();
		listaEstados[JUGAR].onSet();
		estadoActual = JUGAR;
	}
	
	public void nuevoEstadoFinPartida(int puntuacion){
		if(listaEstados[estadoActual] != null){
			listaEstados[estadoActual].onChange();
		}	
		listaEstados[FIN_PARTIDA] = new EstadoFinPartida(puntuacion);
		listaEstados[FIN_PARTIDA].onSet();
		estadoActual = FIN_PARTIDA;		
	}
	
	//Setter estado
	public void setState(int estado){
		if(listaEstados[estadoActual] != null){
			//Avisamos al estado actual del cambio
			listaEstados[estadoActual].onChange();
		}	
		//Avisamos al estado de que le vamos a establecer
		listaEstados[estado].onSet();
		estadoActual = estado;		
	}	
	
	//Actualiza el estado actual
	public void update(){
		if(listaEstados[estadoActual] != null){
			listaEstados[estadoActual].update();
		}
		
	}
	//Renderiza el estado actual
	public void render(Graphics g){
		if(listaEstados[estadoActual] != null){
			listaEstados[estadoActual].render(g);
		}
	}	
	
}
