package SpaceInvaders.GameplayObjects;

import java.awt.image.BufferedImage;
import java.io.IOException;

import javax.imageio.ImageIO;

public class Animacion {

	private int numFrames;
	private BufferedImage[] listaFrames;
	
	private int frameActual;
	private long ultimaActualizacion;
	private long tiempoEntreActualizaciones;
	
	private boolean playedOnce;
	
	
	public Animacion(long tiempoEntreActualizaciones,int numFrames,String path) {
		this.tiempoEntreActualizaciones = tiempoEntreActualizaciones;		
		this.numFrames = numFrames;
		cargarAnimacion(path);
		playedOnce = false;
		this.ultimaActualizacion = System.currentTimeMillis();		
	}	
	
	private void cargarAnimacion(String path){
		BufferedImage img;
		BufferedImage aux;
		listaFrames = new BufferedImage[numFrames];
		
		try {
			//Cargamos la imagen con los diferentes sprites que componen la animacin
			img = ImageIO.read(this.getClass().getResourceAsStream(path));
			int posX = 0;
			int ancho = img.getWidth()/numFrames;
			int alto = img.getHeight();				
			
			//Recorremos la imagen y recortamos los sprites
			for (int i = 1; i <= numFrames; i++) {
				aux = img.getSubimage(posX, 0, ancho, alto);
				listaFrames[i-1] = aux;
				posX += ancho;				
			}
		} catch (IOException e) {			
			e.printStackTrace();
		}		
	}
	
	public boolean getPlayedOnce(){
		return playedOnce;
	}
	
	public BufferedImage getImagen(){
		return listaFrames[frameActual];		
	}
	
	public void update(){
		//Comprobamos si ha pasado el tiempo
		if((System.currentTimeMillis() - ultimaActualizacion) > tiempoEntreActualizaciones){
			//Actualizamos el ndice			
			frameActual++;
			if(frameActual >= numFrames){
				frameActual = 0;
				playedOnce = true;
			}
			ultimaActualizacion = System.currentTimeMillis();
		}
	}	
}
