package SpaceInvaders.GameplayObjects.Enemigos;

import java.awt.Dimension;


import SpaceInvaders.Estados.EstadoJugar;
import SpaceInvaders.GameplayObjects.Animacion;
import SpaceInvaders.GameplayObjects.Bala;
import SpaceInvaders.GameplayObjects.Explosion;

import SpaceInvaders.Utilidades.GestorXML;

public class EnemigoA extends Enemigo {		
	
	//Variables para el sistema disparo	
	private double tiempoEntreDisparos;
	private double ultimoDisparo;
	private double velocidadBala;
	private int anchoBala;
	private int altoBala;
	
	private Animacion animacion;

	public EnemigoA(double posX, double posY,EstadoJugar partida) {
		super(posX, posY,partida);
		
		String s = GestorXML.getGestorXML().obtenerValorConfiguracion("enemigoA", "porcentajeRecompensa");
		this.porcentajeRecompensa = Double.parseDouble(s);
		
		//Ponemos la direccion hacia abajo
		setDireccion(0, 1);	
		iniciarVariablesDisparo();
		cargarAnimacion();
	}	
	
	protected void setPuntos(){
		String s = GestorXML.getGestorXML().obtenerValorConfiguracion("enemigoA","puntos");
		this.puntos = Integer.parseInt(s);
	}
	private void cargarAnimacion(){
		animacion = new Animacion(300L, 2, "/Imagenes/Enemigo1.png");
		this.imagen = animacion.getImagen();
	}
	
	private void iniciarVariablesDisparo() {		
		this.ultimoDisparo = 0;
		//Establecemos el tiempo entre disparos de forma random
		String s = GestorXML.getGestorXML().obtenerValorConfiguracion("enemigoA", "maxTiempoRecarga");
		double max = Double.parseDouble(s);
		s = GestorXML.getGestorXML().obtenerValorConfiguracion("enemigoA", "minTiempoRecarga");
		double min = Double.parseDouble(s);
		this.tiempoEntreDisparos = getValorRandom(min, max);
		
		//Cargamos el resto valores
		s = GestorXML.getGestorXML().obtenerValorConfiguracion("enemigoA", "velocidadBala");
		this.velocidadBala = Double.parseDouble(s);
		
		s = GestorXML.getGestorXML().obtenerValorConfiguracion("enemigoA", "anchoBala");
		this.anchoBala = Integer.parseInt(s);
		
		s = GestorXML.getGestorXML().obtenerValorConfiguracion("enemigoA", "altoBala");
		this.altoBala = Integer.parseInt(s);
	}

	//Establecemos un tamao random
	protected void setSize() {		
		String s = GestorXML.getGestorXML().obtenerValorConfiguracion("enemigoA", "maxSize");
		int max = Integer.parseInt(s);
		s = GestorXML.getGestorXML().obtenerValorConfiguracion("enemigoA", "minSize");
		int min = Integer.parseInt(s);
		int randomSize = (int) getValorRandom(min, max);
		this.ancho = randomSize;
		this.alto = randomSize;		
	}
	//Establecemos velocidad random
	protected void setRandomSpeed() {		
		String s = GestorXML.getGestorXML().obtenerValorConfiguracion("enemigoA", "maxVelocidad");
		double maxV = Double.parseDouble(s);
		s = GestorXML.getGestorXML().obtenerValorConfiguracion("enemigoA", "minVelocidad");
		double minV = Double.parseDouble(s);
		this.velocidadActual = getValorRandom(minV, maxV);		
	}	
	//Establece la posicion inicial
	protected void setPosicionInicial() {
		Dimension screenSize = partida.getDimensionPantalla();
		this.posY = 0 - this.alto;
		this.posX = getValorRandom(0+this.ancho, screenSize.getWidth()-this.ancho);		
	}
	
	
	private void disparar(){
		//Comprobamos si se puede disparar
		if((System.currentTimeMillis() - ultimoDisparo) > tiempoEntreDisparos){
			double x = this.posX + this.ancho/2;
			double y = this.posY + this.alto;
			Bala b = new Bala(x, y, Bala.BALA_ENEMIGO, anchoBala,altoBala,"/Imagenes/Bullet2.png", 
					this.partida,velocidadBala,0,1);
			this.partida.addBala(b);			
			ultimoDisparo = System.currentTimeMillis();
		}
	}		
	
	//Unicamente comprobamos si se va por abajo
	public void fueraPantalla(Dimension d) {
		//Comprobamos abajo
		if((posY - alto) > d.getHeight()){
			fin = true;			
			return;
		}
	}
	
	public void update() {
		if(!fin){
			//Actualizamos la animacion
			animacion.update();
			//Actualizamos la imagen a dibujar
			this.imagen = animacion.getImagen();
			//Movemos
			move();
			//Comprobamos colision jugador
			comprobarColisionJugador();
			//Comprobamos si estamos fuera
			fueraPantalla(partida.getDimensionPantalla());
			//Intentamos disparar
			disparar();
		}				
	}
	
	protected void crearExplosion() {
		Explosion e = new Explosion(this.posX, this.posY, partida,this.alto);
		partida.addExplosion(e);		
	}
}
