package SpaceInvaders.GameplayObjects.Enemigos;

import java.awt.Dimension;

import javax.imageio.ImageIO;

import SpaceInvaders.Estados.EstadoJugar;
import SpaceInvaders.GameplayObjects.Bala;
import SpaceInvaders.GameplayObjects.Explosion;

import SpaceInvaders.Utilidades.GestorXML;

public class EnemigoC extends Enemigo {
	
	//Variables para el sistema disparo	
	private double tiempoEntreDisparos;
	private double ultimoDisparo;
	private double velocidadBala;
	private int anchoBala;
	private int altoBala;

	public EnemigoC(double posX, double posY,EstadoJugar partida) {
		super(posX, posY,partida);		
		String s = GestorXML.getGestorXML().obtenerValorConfiguracion("enemigoC", "porcentajeRecompensa");
		this.porcentajeRecompensa = Double.parseDouble(s);
		iniciarVariablesDisparo();
		cargarImagen();		
	}	
	
	private void cargarImagen(){
		try {
			imagen = ImageIO.read(this.getClass().getResourceAsStream("/Imagenes/Enemigo3.png"));
		} catch (Exception e) {
			e.printStackTrace();
		}
	}	
	
	protected void setPuntos(){
		String s = GestorXML.getGestorXML().obtenerValorConfiguracion("enemigoC","puntos");
		this.puntos = Integer.parseInt(s);
	}
	
	private void iniciarVariablesDisparo() {		
		this.ultimoDisparo = 0;
		//Establecemos el tiempo entre disparos de forma random
		String s = GestorXML.getGestorXML().obtenerValorConfiguracion("enemigoC", "maxTiempoRecarga");
		double max = Double.parseDouble(s);
		s = GestorXML.getGestorXML().obtenerValorConfiguracion("enemigoC", "minTiempoRecarga");
		double min = Double.parseDouble(s);
		this.tiempoEntreDisparos = getValorRandom(min, max);
		
		//Cargamos el resto valores
		s = GestorXML.getGestorXML().obtenerValorConfiguracion("enemigoC", "velocidadBala");
		this.velocidadBala = Double.parseDouble(s);
		
		s = GestorXML.getGestorXML().obtenerValorConfiguracion("enemigoC", "anchoBala");
		this.anchoBala = Integer.parseInt(s);
		
		s = GestorXML.getGestorXML().obtenerValorConfiguracion("enemigoC", "altoBala");
		this.altoBala = Integer.parseInt(s);
	}
	
	protected void setPosicionInicial() {
		Dimension d = partida.getDimensionPantalla();				
		//Obtenemos la posicion y random
		String s = GestorXML.getGestorXML().obtenerValorConfiguracion("enemigoC","posYMaxima");
		this.posY = getValorRandom(0, Double.parseDouble(s));		
		//Obtenemos de forma random si aparece izquierda o derecha
		double r = getValorRandom(0, 1);
		if(r >= 0.5){
			//Iniciamos en la derecha moviendonos a izquierda
			this.posX = d.getWidth();
			setDireccion(-1, 0);			
		}else			
		{	//Iniciamos en la izquierda moviendonos a derecha
			this.posX = -this.ancho;
			setDireccion(1, 0);
		}		
	}
	
	protected void setSize() {
		String s = GestorXML.getGestorXML().obtenerValorConfiguracion("enemigoC", "ancho");
		this.ancho = Integer.parseInt(s);
		s = GestorXML.getGestorXML().obtenerValorConfiguracion("enemigoC", "alto");
		this.alto = Integer.parseInt(s);		
	}
	
	protected void setRandomSpeed() {
		String s = GestorXML.getGestorXML().obtenerValorConfiguracion("enemigoC", "maxVelocidad");
		double maxV = Double.parseDouble(s);
		s = GestorXML.getGestorXML().obtenerValorConfiguracion("enemigoC", "minVelocidad");
		double minV = Double.parseDouble(s);
		this.velocidadActual = getValorRandom(minV, maxV);				
	}	
	
	private void disparar(){
		//Comprobamos si se puede disparar
		if((System.currentTimeMillis() - ultimoDisparo) > tiempoEntreDisparos){
			//Obtenemos la posicion inicial de la bala y la creamos
			double x = this.posX + this.ancho/2;
			double y = this.posY + this.alto;
			Bala b = new Bala(x, y, Bala.BALA_ENEMIGO, anchoBala,altoBala,"/Imagenes/Bullet3.png", 
					this.partida,velocidadBala,0,1);
			this.partida.addBala(b);			
			ultimoDisparo = System.currentTimeMillis();
		}	
	}
	
	public void fueraPantalla(Dimension d) {
		//Comprobamos si ha chocado con la izquierda
		if(posX  < 0){
			//Cambiamos direccion a derecha
			setDireccion(1, 0);		
			return;
		}
		//Comprobamos si ha chocado con la derecha
		if((posX + ancho)> d.getWidth()){
			//Cambiamos direccion a izquierda
			setDireccion(-1, 0);
			return;
		}
	}
	
	public void update() {
		if(!fin){			
			//Movemos
			move();
			//Comprobamos si estamos fuera
			fueraPantalla(partida.getDimensionPantalla());
			//Intentamos disparar
			disparar();
		}		
	}
	
	protected void crearExplosion() {
		Explosion e = new Explosion(this.posX+(this.ancho/2), this.posY, partida,this.alto);
		partida.addExplosion(e);
		
	}

}
