package SpaceInvaders.GameplayObjects.Enemigos;

import java.awt.Dimension;


import SpaceInvaders.Estados.EstadoJugar;
import SpaceInvaders.Utilidades.GestorXML;

public class SistemaSpawn {

	private EstadoJugar partida;
	
	private int oleadaActual;
	private int duracionOleada;
	
	private Dimension screenSize;
	
	//Variables para controlar el spawn de cada enemigo
	private int totalEnemigosA;
	private int enemigosCreadosA;
	private long ultimoSpawnA;
	private long tiempoEntreSpawnA;
	
	private int totalEnemigosB;
	private int enemigosCreadosB;
	private long ultimoSpawnB;
	private long tiempoEntreSpawnB;
	
	private int totalEnemigosC;
	private int enemigosCreadosC;
	private long ultimoSpawnC;
	private long tiempoEntreSpawnC;	
	
	
	public SistemaSpawn(EstadoJugar partida) {
		this.partida = partida;
		this.screenSize = partida.getDimensionPantalla();
		this.oleadaActual = 1;
		String s = GestorXML.getGestorXML().obtenerValorConfiguracion("sistemaSpawn", "duracionOleada");
		this.duracionOleada = Integer.parseInt(s);
		
		iniciarOleada();
	}
	
	public void update(){
		//Intenta crear los diferentes enemigos
		spawnEnemigoA();
		spawnEnemigoB();
		spawnEnemigoC();		
		
	}
	
	private void iniciarOleada(){	
		//Establecemos las variables para spawnear cada enemigo de acuerdo a una formula
		this.enemigosCreadosA = 0;
		this.ultimoSpawnA = 0;
		this.totalEnemigosA = 10*oleadaActual;
		this.tiempoEntreSpawnA = duracionOleada/totalEnemigosA;		
		
		this.enemigosCreadosB = 0;
		this.ultimoSpawnB = 0;
		this.totalEnemigosB = 5 + (2*oleadaActual);
		this.tiempoEntreSpawnB = duracionOleada/totalEnemigosB;
		
		this.enemigosCreadosC = 0;
		this.ultimoSpawnC = 0;
		this.totalEnemigosC = 2*oleadaActual;
		this.tiempoEntreSpawnC = duracionOleada/totalEnemigosC;
	}
	
	public void nuevaOleda(){
		oleadaActual++;
		iniciarOleada();
	}	
	
	
	private void spawnEnemigoA(){
		//Comprobamos si ya hemos spawneados el total de enemigos de tipo A para esta oleada
		if(enemigosCreadosA >= totalEnemigosA){
			return;
		}		
		//Comprobamos que haya pasado el tiempo para volver a spawnear
		if((System.currentTimeMillis() - ultimoSpawnA) > tiempoEntreSpawnA){
			//Creamos un enemigo tipo A			
			EnemigoA e = new EnemigoA(0, 0, partida);
			partida.addEnemigo(e);			
			//Actualizamos el ndice
			this.ultimoSpawnA = System.currentTimeMillis();
			enemigosCreadosA++;
		}
	}
	
	private void spawnEnemigoB(){
		//Comprobamos si ya hemos spawneados el total de enemigos de tipo B para esta oleada
		if(enemigosCreadosB >= totalEnemigosB){
			return;
		}
		//Comprobamos que haya pasado el tiempo para volver a spawnear
		if((System.currentTimeMillis() - ultimoSpawnB) > tiempoEntreSpawnB){
			//Creamos un enemigo tipo B			
			EnemigoB e = new EnemigoB(0,0, partida);
			partida.addEnemigo(e);			
			//Actualizamos el ndice
			this.ultimoSpawnB = System.currentTimeMillis();
			enemigosCreadosB++;
		}
	}

	private void spawnEnemigoC(){
		//Comprobamos si ya hemos spawneados el total de enemigos de tipo C para esta oleada
		if(enemigosCreadosC >= totalEnemigosC){
			return;
		}
		//Comprobamos que haya pasado el tiempo para volver a spawnear
		if((System.currentTimeMillis() - ultimoSpawnC) > tiempoEntreSpawnC){
			//Creamos un enemigo tipo C			
			EnemigoC e = new EnemigoC(0,0, partida);
			partida.addEnemigo(e);			
			//Actualizamos el ndice
			this.ultimoSpawnC = System.currentTimeMillis();
			enemigosCreadosC++;
		}
	}

	public boolean finOleada() {
		//Devuelve true si se han creado todos los enemigos de cada tipo para esta oleada
		return enemigosCreadosA == totalEnemigosA && enemigosCreadosB == totalEnemigosB 
				&& enemigosCreadosC == totalEnemigosC;
	}
	
}
