package SpaceInvaders.GameplayObjects;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;

import SpaceInvaders.Estados.EstadoJugar;

public abstract class GameObject {

	protected double posX;
	protected double posY;
	
	protected int ancho;
	protected int alto;
	
	protected double velocidadActual;
	protected double dX;
	protected double dY;
	
	protected BufferedImage imagen;	
	
	protected EstadoJugar partida;
	
	protected boolean fin;
	
	//Constructor
	public GameObject(double posX, double posY,EstadoJugar partida) {		
		this.posX = posX;		
		this.posY = posY;
		this.partida = partida;
		this.fin = false;
	}

	public Rectangle getColisionBox(){
		return new Rectangle((int)posX,(int)posY,ancho,alto);
	}
	
	protected boolean hayColision(GameObject go){
		if(!fin){
			return getColisionBox().intersects(go.getColisionBox());
		}
		return false;
		
	}
	
	protected void move(){
		this.posX += velocidadActual*dX;
		this.posY += velocidadActual*dY;
	}
	
	//Normalizar vector de direccion
	protected void setDireccion(double dX,double dY){
		if(dX == 0 && dY == 0){
			this.dX = 0;
			this.dY = 0;
		}else
		{
			double longitud = Math.sqrt(Math.pow(dX, 2) + Math.pow(dY, 2));			
			this.dX = dX/longitud;		
			this.dY = dY/longitud;
		}
		
	}

	
	public boolean getFin(){
		return fin;
	}
	
	protected void fueraPantalla(Dimension d){
		//Comprobamos si se va por la izquierda totalmente
		if((posX + ancho) < 0){
			fin = true;
			return;
		}
		//Comprobamos derecha
		if(posX > d.getWidth()){
			fin = true;
			return;
		}
		//Comprobamos arriba
		if((posY + alto) < 0){
			fin = true;
			return;
		}
		//Comprobamos abajo
		if(posY > d.getHeight()){
			fin = true;			
			return;
		}
	}
	
	public abstract void update();
	
	public void draw(Graphics g){
		if(!fin){			
			g.drawImage(imagen, (int)this.posX, (int)this.posY,ancho,alto,null);
		}		
	}
}
