package SpaceInvaders.GameplayObjects.Jugador;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.image.BufferedImage;


import javax.imageio.ImageIO;


import SpaceInvaders.ControladoresEntrada.ControladorRaton;
import SpaceInvaders.ControladoresEntrada.ControladorTeclado;
import SpaceInvaders.Estados.EstadoJugar;
import SpaceInvaders.Estados.StateMachine;
import SpaceInvaders.GameplayObjects.Bala;
import SpaceInvaders.GameplayObjects.GameObject;
import SpaceInvaders.GameplayObjects.Bonificaciones.Bonificacion;
import SpaceInvaders.GameplayObjects.Bonificaciones.GestorBonificaciones;
import SpaceInvaders.GestorJuego.GestorPrincipal;
import SpaceInvaders.Utilidades.GestorXML;

public class Jugador extends GameObject {
	
	
	private double velocidadInicial;
	
	//Variables para controlar el sistema de disparo
	private long tiempoEntreDisparosActual;
	private long tiempoEntreDisparosInicial;
	private double velocidadBala;
	private int anchoBala;
	private int altoBala;
	private long ultimoDisparo;	
	
	//Bonificaciones
	private GestorBonificaciones gestorBonificaciones;
	
	//Barra vida y puntuaciones
	private GestorPuntuacion gestorPuntuacion;
	private GestorVida gestorVida;

	private boolean escudo;
	private BufferedImage imgEscudo;
	
	private boolean tripleDisparo;
	
	

	public Jugador(double posX, double posY,EstadoJugar partida) {
		super(posX, posY,partida);
		
		this.escudo = false;
		this.tripleDisparo = false;
		
		gestorBonificaciones = new GestorBonificaciones();
		gestorVida = new GestorVida(this);
		gestorPuntuacion = new GestorPuntuacion();
		
		cargarImagen();
		
		//Cargamos variables del fichero configuracin
		//Ponemos la velocidad inicial
		String s = GestorXML.getGestorXML().obtenerValorConfiguracion("naveJugador","velocidadInicial");
		this.velocidadInicial = Double.parseDouble(s);
		this.velocidadActual = velocidadInicial;		
		
		//Dimensiones
		s = GestorXML.getGestorXML().obtenerValorConfiguracion("naveJugador","ancho");
		this.ancho = Integer.parseInt(s);
		s = GestorXML.getGestorXML().obtenerValorConfiguracion("naveJugador","alto");
		this.alto = Integer.parseInt(s);
		
		//Direccin inicial
		setDireccion(0, 0);
		
		//Variables para el sistema de tiro
		s = GestorXML.getGestorXML().obtenerValorConfiguracion("naveJugador", "tiempoEntreDisparosInicial");
		this.tiempoEntreDisparosInicial = Long.parseLong(s);
		this.tiempoEntreDisparosActual = tiempoEntreDisparosInicial;
		this.ultimoDisparo = 0L;
		
		s = GestorXML.getGestorXML().obtenerValorConfiguracion("naveJugador", "velocidadBala");
		this.velocidadBala = Double.parseDouble(s);
		
		s = GestorXML.getGestorXML().obtenerValorConfiguracion("naveJugador", "altoBala");
		this.altoBala = Integer.parseInt(s);	
		
		s = GestorXML.getGestorXML().obtenerValorConfiguracion("naveJugador", "anchoBala");
		this.anchoBala = Integer.parseInt(s);		
		
	}
	
	private void cargarImagen() {
		try {
			imagen = ImageIO.read(this.getClass().getResourceAsStream("/Imagenes/Nave.png"));
			imgEscudo = ImageIO.read(this.getClass().getResourceAsStream("/Imagenes/Escudo.png"));
		} catch (Exception e) {
			e.printStackTrace();
		}		
	}

	public void addBonificacion(Bonificacion b){
		gestorBonificaciones.addBonificacion(b);
	}
	
	public double getPosicionX(){
		return posX;
	}
	
	public double getPosicionY(){
		return posY;
	}
	
	public void setTiempoEntreDisparos(long tiempo){		
		this.tiempoEntreDisparosActual = tiempo;
	}
	
	public void setTiempoEntreDisparosInicial(){		
		this.tiempoEntreDisparosActual = tiempoEntreDisparosInicial;
	}
	
	//Devuelve la puntuacin obtenida
	public int getPuntuacion(){
		return gestorPuntuacion.getPuntuacion();
	}
	
	public void addPuntuacion(int puntos){
		gestorPuntuacion.addPuntuacion(puntos);
	}
	
	//Aadir y quitar vidas
	public void quitarVida(){
		//Si tiene escudo no quitamos vida
		if(!escudo){
			gestorVida.quitarVida();
		}		
	}
	
	public void addVida() {
		gestorVida.addVida();
	}
	
	//Escudo
	public void setEscudo(boolean b){		
		this.escudo = b;
	}
	
	//TripleDisparo
	public void setTripleDisparo(boolean b){
		this.tripleDisparo = b;
	}
	
	public void update() {
		if(!fin){
			//Comprobamos los inputs
			checkInputs();		
			//Movemos
			move();
			//Comprobamos si estamos fuera del lmite
			fueraPantalla(partida.getDimensionPantalla());	
			//Actualizamos el gestor bonificiones
			gestorBonificaciones.update();
		}		
	}	
	
	public void muerto(){
		partida.jugadorMuerto();
	}
	
	
	public void fueraPantalla(Dimension d) {
		//Comprobamos limite izquierdo
		if(this.posX < 0){
			this.posX = 0;			
		}
		//Comprobamos limite derecho
		if((this.posX + ancho ) > d.getWidth()){
			this.posX = d.getWidth() - ancho;			
		}
		
	}
	
	private void disparar(Point posRaton){		
		//Comprobamos si puede disparar
		if((System.currentTimeMillis() - ultimoDisparo) > tiempoEntreDisparosActual){			
			//Calculamos la direccin y posicion iniciald
			double x = this.posX + (ancho/2 - 6);
			double y = this.posY ;			
			double dX = posRaton.getX() - x;			
			double dY = posRaton.getY() - y;
			//Las balas no pueden ir para abajo
			if(dY > 0){
				dY = 0;
			}
			//Comprobamos si esta activo el triple disparo
			if(tripleDisparo){
				Bala b = new Bala(x,y, Bala.BALA_JUGADOR, anchoBala, altoBala,
						"/Imagenes/Bullet1.png", this.partida, velocidadBala, dX+40, dY);
				Bala b2 = new Bala(x,y, Bala.BALA_JUGADOR, anchoBala, altoBala,
						"/Imagenes/Bullet1.png", this.partida, velocidadBala, dX-40, dY);
				Bala b3 = new Bala(x,y, Bala.BALA_JUGADOR, anchoBala, altoBala,
						"/Imagenes/Bullet1.png", this.partida, velocidadBala, dX, dY);
				partida.addBala(b);
				partida.addBala(b2);
				partida.addBala(b3);
				ultimoDisparo = System.currentTimeMillis();
				
			}else{
				//Creamos la bala y aadimos a la lista de la partida
				Bala b = new Bala(x,y, Bala.BALA_JUGADOR, anchoBala, altoBala,
						"/Imagenes/Bullet1.png", this.partida, velocidadBala, dX, dY);
				partida.addBala(b);			
				ultimoDisparo = System.currentTimeMillis();
			}
			
		}
	}
	
	
	public void draw(Graphics g) {		
		super.draw(g);
		if(!fin){
			//Dibujamos puntuacion y vida
			gestorPuntuacion.draw(g);
			gestorVida.draw(g);	
			if(escudo){
				dibujarEscudo(g);
			}
		}		
	}
	
	private void dibujarEscudo(Graphics g) {
		g.drawImage(imgEscudo, (int)this.posX, (int)this.posY,ancho,alto,null);		
	}

	private void checkInputs(){
		//Comprobamos si estan pulsadas las teclas de movimiento
		if(ControladorTeclado.getControladorTeclado().estaPulsada(ControladorTeclado.DERECHA)){
			//Ponemos direccin derecha
			setDireccion(1, 0);
			
		}else if(ControladorTeclado.getControladorTeclado().estaPulsada(ControladorTeclado.IZQUIERDA)){
			//Ponemos direccin izquierda
			setDireccion(-1, 0);
		}else
		{
			//Si no hay direccin para la nave
			setDireccion(0, 0);
		}		
		//Si el usuario ha presionado el click izquierdo disparamos
		if(ControladorRaton.getControladorRaton().getClickIzquierdoPresionado()){
			disparar(ControladorRaton.getControladorRaton().getPosicion());
		}
		//Comprobmaos si pulsa pausa
		if(ControladorTeclado.getControladorTeclado().haSidoPulsada(ControladorTeclado.P)){
			GestorPrincipal.getGestorPrincipal().cambiarEstado(StateMachine.PAUSA);
		}
		//Comprobamos si pulsa Esc
		if(ControladorTeclado.getControladorTeclado().haSidoPulsada(ControladorTeclado.ESC)){
			//Finalizamos la partida
			muerto();
		}
	}

	
	

}
