package SpaceInvaders.GestorJuego;
import java.awt.Canvas;

import java.awt.Dimension;
import java.awt.Graphics;

import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.image.BufferStrategy;

import javax.swing.SwingUtilities;

import SpaceInvaders.ControladoresEntrada.ControladorRaton;
import SpaceInvaders.ControladoresEntrada.ControladorTeclado;



public class GameCanvas extends Canvas implements Runnable{

	private Thread thread;
	private boolean activo;
	private VentanaPrincipal ventana;
	private ContoladorEntrada controladorEntrada;
	
	public GameCanvas() {
		//Creamos el JFrame y le pasamos este canvas
		ventana = new VentanaPrincipal(this);
		//Creamos los controladores de entrada por teclado y raton
		controladorEntrada = new ContoladorEntrada();
		this.addKeyListener(controladorEntrada);
		this.addMouseListener(controladorEntrada);
		this.addMouseMotionListener(controladorEntrada);
		//Ponemos el focus en el canvas 
		this.setFocusable(true);			
	}
	
	//Iniciamos el juego
	public synchronized void iniciarJuego(){
		if(thread == null){
			thread = new Thread(this);
			thread.start();
			activo = true;
		}
	}
	
	//Paramos el juego
	public synchronized void pararJuego(){
		ventana.cerrarVentana();
		try {
			thread.stop();
			activo = false;			
		} catch (Exception e) {
			e.printStackTrace();
		}		
	}		
	
	//Game loop
	public void run() {
		
		long lastTime = System.nanoTime();
		double amountOfTicks = 60.0;
		double ns = 1000000000 / amountOfTicks;
		double delta = 0;
		long timer = System.currentTimeMillis();
		long timer2 = System.currentTimeMillis();
		int frames = 0;			
		int aps = 0;
		
		while(activo){
			long now = System.nanoTime();
			delta += (now - lastTime) / ns;
			lastTime = now;
			while(delta >= 1){
				aps++;
				updateGame();				
				delta--;				
			}
			if(activo){
				renderGame();
				frames++;
			}			
			if(System.currentTimeMillis() - timer2 > 1000){
				timer2 +=1000;
				System.out.println("APS: " + aps);
				aps = 0;
			}	
			if(System.currentTimeMillis() - timer > 1000){
				timer +=1000;
				System.out.println("FPS: " + frames);
				frames = 0;
			}			
		}
		//Si sale del loop paramos el juego
		pararJuego();
		
	}
	
	//Actualiza el juego
	private void updateGame() {
		GestorPrincipal.getGestorPrincipal().update();		
		//Actualizamos los controladores
		ControladorRaton.getControladorRaton().actualizar();
		ControladorTeclado.getControladorTeclado().actualizarTeclado();		
	}

	//Renderiza la pantalla
	private void renderGame() {
		BufferStrategy bs = this.getBufferStrategy();
		if(bs == null){
			this.createBufferStrategy(3);
			return;
		}		
		Graphics g = bs.getDrawGraphics();	
		Dimension d = ventana.getDimensionPantalla();		
		g.clearRect(0, 0, d.width,d.height);		
		GestorPrincipal.getGestorPrincipal().render(g);			
		g.dispose();
		bs.show();
	}
	
	private class ContoladorEntrada implements KeyListener,MouseListener,MouseMotionListener{
		
		//Le pasamos al gestor del teclado la tecla presionada
		public void keyPressed(KeyEvent k) {			
			ControladorTeclado.getControladorTeclado().teclaPresionada(k.getKeyCode());
		}		
		//Le pasamos al gestor del teclado la tecla liberada
		public void keyReleased(KeyEvent k) {			
			ControladorTeclado.getControladorTeclado().teclaLiberada(k.getKeyCode());		
		}		
		public void keyTyped(KeyEvent k) {			
			
		}
		//Gestin del click del raton
		public void mousePressed(MouseEvent e) {
			//Comprobamos que sea click izquierdo
			if(SwingUtilities.isLeftMouseButton(e)){				
				ControladorRaton.getControladorRaton().setClickIzquierdo(true);				
			}
		}
		//Evento para gestionar el movimiento de raton
		public void mouseMoved(MouseEvent e) {	
			//Establemos la posicin en el controlador
			ControladorRaton.getControladorRaton().setPosicion(e.getX(), e.getY());			
		}		
		
		public void mouseClicked(MouseEvent e) {}		
		public void mouseEntered(MouseEvent e) {}		
		public void mouseExited(MouseEvent e) {}				
		public void mouseReleased(MouseEvent e) {}		
		public void mouseDragged(MouseEvent e) {}		
	}
}
